@@ -117,7 +117,7 @@ let shorten str limit =
     with e -> slen
   in
   let diff_len_utf8_ascii = slen - len in
-  let max_len = maxi limit 10 in
+  let max_len = max limit 10 in
   if len > max_len then
     let prefix = String.sub name 0 (max_len - 7 + diff_len_utf8_ascii) in
     let suffix = String.sub name (len - 4 + diff_len_utf8_ascii) 4 in
@@ -177,7 +177,7 @@ let connection_failed cc =
   cc.control_state <- cc.control_state + 1
 
 let connection_next_try cc =
-  cc.control_last_try + mini (cc.control_min_reask * cc.control_state)
+  cc.control_last_try + min (cc.control_min_reask * cc.control_state)
   cc.control_min_reask
 
 let connection_can_try cc =
@@ -213,20 +213,37 @@ let download_control = TcpBufferedSocket.create_read_bandwidth_controler
 
 let payload_bandwidth = ref 0.
 
+let check_ul_dl_ratio () =
+  if !!max_hard_upload_rate < 0 then max_hard_upload_rate =:= 0;
+  if !!max_hard_download_rate < 0 then max_hard_download_rate =:= 0;
+  let max_max_hard_download_rate =
+    match !!max_hard_upload_rate with
+    | 0 -> None
+    | x when x < 4 -> Some (x * 3)
+    | x when x < 10 -> Some (x * 4)
+    | x -> None in
+  match max_max_hard_download_rate with
+  | None -> ()
+  | Some limit ->
+      if !!max_hard_download_rate = 0 ||
+	!!max_hard_download_rate > limit then
+	  max_hard_download_rate =:= limit
+
 let _ =
   option_hook max_hard_upload_rate (fun _ ->
+      check_ul_dl_ratio ();
       TcpBufferedSocket.change_rate upload_control
         (!!max_hard_upload_rate * 1024);
       payload_bandwidth :=
 	float_of_int (if !!max_hard_upload_rate = 0 then 
 	   10000 * 1024
          else 
-	   maxi (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
+	   max (!!max_hard_upload_rate * 1024) 1024) *. 0.90;
   );
   option_hook max_hard_download_rate (fun _ ->
-      let rate = !!max_hard_download_rate in
+      check_ul_dl_ratio ();
       TcpBufferedSocket.change_rate download_control
-        (rate * 1024))
+        (!!max_hard_download_rate * 1024))
 
 let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
@@ -464,17 +481,18 @@ let chat_message_fifo = (Fifo.create () : (int * string * int * string * string)
 
 let log_chat_message i num n s =
   Fifo.put chat_message_fifo (last_time(),i,num,n,s);
-  try
+  (try
     Unix2.tryopen_write_gen !messages_log [Open_creat; Open_wronly; Open_append] 
       0o600 (fun oc ->
 	Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s)
   with e ->
     lprintf_nl "[ERROR] Exception %s while trying to log message to %s"
-      (Printexc2.to_string e) !messages_log;
+      (Printexc2.to_string e) !messages_log);
 
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages do
     ignore(Fifo.take chat_message_fifo)
   done
+
 let last_message_log = ref 0
 
 
@@ -482,7 +500,7 @@ let last_message_log = ref 0
 let debug_clients = ref Intset.empty
 
 let default_user = {
-    ui_user_name = CommonUserDb.admin_user;
+    ui_user = CommonUserDb.admin_user;
     ui_user_searches = [];
     ui_last_search = None;
     ui_last_results = [];
@@ -496,7 +514,7 @@ let find_ui_user user =
     match list with
       [] ->
         let u = {
-            ui_user_name = user;
+            ui_user = (CommonUserDb.user2_user_find user);
             ui_user_searches = [];
             ui_last_search = None;
             ui_last_results = [];
@@ -505,7 +523,7 @@ let find_ui_user user =
         ui_users := u :: !ui_users;
         u
     | u :: tail ->
-        if u.ui_user_name = user then u else iter tail
+        if u.ui_user = (CommonUserDb.user2_user_find user) then u else iter tail
   in
   iter !ui_users
 
@@ -600,8 +618,13 @@ let update_link_stats () =
   put time sample short_delay_bandwidth_samples;
   trimto 5 short_delay_bandwidth_samples
 
+let history_step = 5
 let history_size = 720
-let history_h_size = 720
+let history_size_for_h_graph = ref 0 (* history_size * !!html_mods_vd_gfx_h_intervall / 60 *)
+let history_h_step = ref 0 (* 60 * !!html_mods_vd_gfx_h_intervall <= history_size * history_step *)
+let history_h_size = 1440
+let history_timeflag = ref 0.
+let history_h_timeflag = ref 0.
 
 let upload_history = Fifo.create ()
 let download_history = Fifo.create ()
@@ -620,7 +643,7 @@ let download_usage () =
 let update_download_history () =
   Fifo.put download_history (download_usage ());
   let len = ref (Fifo.length download_history) in
-  while !len > history_size do
+  while !len > history_size+1 do
     ignore (Fifo.take download_history);
     decr len
   done
@@ -628,35 +651,46 @@ let update_download_history () =
 let update_upload_history () =
   Fifo.put upload_history (upload_usage ());
   let len = ref (Fifo.length upload_history) in
-  while !len > history_size do
+  while !len > history_size+1 do
     ignore (Fifo.take upload_history);
     decr len
   done
 
 let update_h_download_history () =
-  Fifo.put download_h_history ((List.fold_left (+) 0 (Fifo.to_list download_history)) / ((Fifo.length download_history)));
+  if (!history_h_step = history_size * history_step) or ( (Fifo.length download_history) <= (!history_size_for_h_graph+1)) then 
+    Fifo.put download_h_history ((List.fold_left (+) 0 (Fifo.to_list download_history)) / ((Fifo.length download_history))) 
+  else 
+    Fifo.put download_h_history ((List.fold_left (+) 0 
+		  (snd (List2.cut ((Fifo.length download_history) - !history_size_for_h_graph - 1) (Fifo.to_list download_history)))) 
+      / (!history_size_for_h_graph + 1) );
   let len = ref (Fifo.length download_h_history) in
-  while !len > history_h_size do
+  while !len > history_h_size+1 do
     ignore (Fifo.take download_h_history);
     decr len
   done
 
 let update_h_upload_history () =
-  Fifo.put upload_h_history ((List.fold_left (+) 0 (Fifo.to_list upload_history)) / ((Fifo.length upload_history)));
+	(* Fifo.put upload_h_history ((Fifo.length upload_history) / 720); *)
+  if (!history_h_step = history_size * history_step) or ((Fifo.length upload_history) <= (!history_size_for_h_graph+1)) then 
+    Fifo.put upload_h_history ((List.fold_left (+) 0 (Fifo.to_list upload_history)) / ((Fifo.length upload_history))) 
+  else 
+    Fifo.put upload_h_history ((List.fold_left (+) 0 
+			(snd (List2.cut ((Fifo.length upload_history) - !history_size_for_h_graph - 1) (Fifo.to_list upload_history)))) 
+      / (!history_size_for_h_graph+1) );
   let len = ref (Fifo.length upload_h_history) in
-  while !len > history_h_size do
+  while !len > history_h_size+1 do
     ignore (Fifo.take upload_h_history);
     decr len
   done
 
 let detected_link_capacity link =
-  List.fold_left maxi 0 (Fifo.to_list link)
+  List.fold_left max 0 (Fifo.to_list link)
 
 let detected_uplink_capacity () =
-  List.fold_left maxi 0 (Fifo.to_list upload_history)
+  List.fold_left max 0 (Fifo.to_list upload_history)
 
 let detected_downlink_capacity () =
-  List.fold_left maxi 0 (Fifo.to_list download_history)
+  List.fold_left max 0 (Fifo.to_list download_history)
 
 
 let new_tag name v =
@@ -873,10 +907,36 @@ let _ =
       activity := new_activity ()
   )
 
-module HashMagic = Weak.Make(struct
+module StringIntern = Weak.Make(struct
         type t = string
         let hash s = Hashtbl.hash s
         let equal x y = x = y
       end)
 
-let files_magic = HashMagic.create 100
+let intern_table = StringIntern.create 1000
+let intern s = StringIntern.merge intern_table s
+
+let print_command_result o buf result =
+  if use_html_mods o then
+    html_mods_table_one_row buf "serversTable" "servers" [
+      ("", "srh", result); ]
+  else
+    Printf.bprintf buf "%s" result
+
+let _ =
+  Heap.add_memstat "CommonGlobals" (fun level buf ->
+      let counter = ref 0 in
+      StringIntern.iter (fun f -> incr counter;) intern_table;
+      Printf.bprintf buf "  intern_table: %d\n" !counter;
+      Printf.bprintf buf " core_gui_fifo: %d\n" (Fifo.length core_gui_fifo);
+      Printf.bprintf buf " gui_core_fifo: %d\n" (Fifo.length gui_core_fifo);
+      Printf.bprintf buf " chat_message_fifo: %d\n" (Fifo.length chat_message_fifo);
+      Printf.bprintf buf " upload_history: %d\n" (Fifo.length upload_history);
+      Printf.bprintf buf " download_history: %d\n" (Fifo.length download_history);
+      Printf.bprintf buf " upload_h_history: %d\n" (Fifo.length upload_h_history);
+      Printf.bprintf buf " download_h_history: %d\n" (Fifo.length download_h_history);
+      Printf.bprintf buf " bandwidth_samples: %d\n" (Fifo.length bandwidth_samples);
+      Printf.bprintf buf " short_delay_bandwidth_samples: %d\n" (Fifo.length short_delay_bandwidth_samples);
+      Printf.bprintf buf " dummy_sample: %d\n" (Array.length dummy_sample);
+      Printf.bprintf buf " activities: %d\n" (Fifo.length activities);
+  )
