@@ -9,7 +9,8 @@ type 'a shared_impl = {
   mutable impl_shared_size : int64;
   mutable impl_shared_id : Md4.Md4.t;
   mutable impl_shared_requests : int;
-  mutable impl_shared_magic : string option
+  mutable impl_shared_magic : string option;
+  mutable impl_shared_servers : CommonTypes.server list;
 } 
 and 'a shared_ops = {
   mutable op_shared_info : 'a -> GuiTypes.shared_info;
@@ -35,6 +36,7 @@ val files_scanned_size : int64 ref
 val new_shared : string -> int -> string -> string -> unit
 val shared_num : CommonTypes.shared -> int
 val shared_fullname : CommonTypes.shared -> string
+val shared_size : CommonTypes.shared -> Int64.t
 val shared_codedname : CommonTypes.shared -> string
 val shared_unshare : CommonTypes.shared -> unit
 val shared_dir : CommonTypes.shared option -> string
