@@ -88,6 +88,9 @@ let file_string_of_uid uid =
   string_of_uid_sep uid "_"
 
 exception Illegal_urn of string
+exception Torrent_already_exists
+exception Torrent_can_not_be_used
+
 let uid_of_string s =
   let s = String.lowercase s in
   let urn = String2.before s 4 in
@@ -287,7 +290,7 @@ let client_friend_tag = 1
 let client_contact_tag = 2
 let client_nolimit_tag = 4
 let client_initialized_tag = 8
-let client_browsed_tag = client_contact_tag lor client_friend_tag
+let client_must_browse_tag = 16
 
 type query_entry = 
   Q_AND of query_entry list
@@ -322,10 +325,10 @@ type host_state =
 | Connected of int    (* >= 0 Queued *)
 | Connected_downloading of int
 (* | ConnectionWaiting of int  *)
-  
 | NewHost
 | RemovedHost
 | BlackListedHost
+| ServerFull
 
 type compressor = 
   Deflate of Zlib.stream * Zlib.stream
@@ -347,7 +350,7 @@ type sharing_strategy = {
 
 type shared_directory = {
     shdir_dirname : string;
-    shdir_priority : int;
+    mutable shdir_priority : int;
     shdir_strategy : string;
     shdir_networks : string list;
   }
@@ -375,6 +378,9 @@ type sortvd_type =
 | ByASources
 | ByNet
 | ByAvail
+| ByComments
+| ByUser
+| ByGroup
 | NotSorted
   
 type room_state = 
@@ -441,17 +447,48 @@ type network_info = {
     network_netnum : int;
     network_config_filename : string;
     network_netflags : network_flag list;
-    mutable network_enabled : bool;
-    mutable network_uploaded : int64;
-    mutable network_downloaded : int64;
-    mutable network_connected : int; (* number of connected servers *)
+    network_enabled : bool;
+    network_uploaded : int64;
+    network_downloaded : int64;
+    network_connected_servers : int;
   }
 
 type extend_search =
   ExtendSearchLocally
 | ExtendSearchRemotely
   
-type network = {
+type network_stat_info  = {
+  mutable string_long : string;
+  mutable string_short : string;
+  mutable seen : int;
+  mutable banned : int;
+  mutable filerequest : int;
+  mutable download : Int64.t;
+  mutable upload : Int64.t;
+}
+
+type network_porttest =
+  PorttestNotAvailable
+| PorttestNotStarted
+| PorttestInProgress of int
+| PorttestResult of int * string
+
+type groupdb = {
+  group_name : string;
+  mutable group_admin : bool;
+}
+
+and userdb = {
+  user_name : string;
+  mutable user_pass : Md4.t;
+  mutable user_groups : groupdb list;
+  mutable user_default_group : groupdb option;
+  mutable user_mail : string;
+  mutable user_commit_dir : string;
+  mutable user_max_concurrent_downloads : int;
+}
+
+and network = {
     network_name : string;
     network_num : int;
     network_connection_manager : TcpBufferedSocket.connection_manager;
@@ -479,7 +516,7 @@ type network = {
     mutable op_network_share : (
       string -> string -> int64 -> unit);
     mutable op_network_private_message : (string -> string -> unit);
-    mutable op_network_parse_url : (string -> string * bool);
+    mutable op_network_parse_url : (string -> userdb -> string * bool);
     mutable op_network_connect_servers : (unit -> unit);
     
     mutable op_network_search : (search -> Buffer.t -> unit);
@@ -491,15 +528,20 @@ type network = {
     mutable op_network_info : (unit -> network_info);
     
     mutable op_network_connected : (unit -> bool);
-    mutable op_network_gui_message : (string -> unit);
+    mutable op_network_gui_message : (string -> userdb -> unit);
     
-    mutable op_network_download : (result_info -> file);
+    mutable op_network_download : (result_info -> userdb -> file);
     mutable op_network_display_stats : (Buffer.t -> ui_conn -> unit);
+    mutable op_network_stat_info_list : unit -> (string * int * (network_stat_info list)) list;
     mutable op_network_clean_exit : (unit -> bool);
+    mutable op_network_reset : (unit -> unit);
+    mutable op_network_ports : (unit -> (int * string) list);
+    mutable op_network_porttest_start : (unit -> unit);
+    mutable op_network_porttest_result : (unit -> network_porttest);
   }
 
-and   ui_user = {
-    ui_user_name : string;
+and ui_user = {
+    ui_user : userdb;
     mutable ui_user_searches : search list;
     mutable ui_last_search : search option;
     mutable ui_last_results : (int * result) list;
@@ -785,6 +827,7 @@ let is_connected state =
   | Connecting
   | NewHost
   | BlackListedHost
+  | ServerFull
   | RemovedHost -> false
 
 let string_of_connection_state s = 
@@ -808,6 +851,7 @@ let string_of_connection_state s =
   | RemovedHost -> "Removed"
   | BlackListedHost -> "Black"
   | NewHost -> "New"
+  | ServerFull -> "Server full"
       
 let short_string_of_connection_state s = 
   match s with
@@ -825,6 +869,7 @@ let short_string_of_connection_state s =
   | RemovedHost -> "Rem"
   | BlackListedHost -> "BL"
   | NewHost -> "New"
+  | ServerFull -> "Full"
     
 exception IgnoreNetwork
   
@@ -884,3 +929,20 @@ type kind_type = {
   f : string -> string -> unit;
   description : string
 }
+
+type slot_kind =
+  NoSlot
+| FriendSlot
+| ReleaseSlot
+| SmallFileSlot
+| NormalSlot
+| PrioSlot of string
+
+let string_of_slot_kind slot_kind short =
+  match slot_kind with
+    NoSlot -> "NoSlot"
+  | FriendSlot -> "FriendSlot"
+  | ReleaseSlot -> "ReleaseSlot"
+  | SmallFileSlot -> "SmallFileSlot"
+  | NormalSlot -> if short then "" else "NormalSlot"
+  | PrioSlot dir -> Printf.sprintf "Prio %s" dir
