@@ -29,6 +29,18 @@ open TcpBufferedSocket
 open CommonGlobals
 open CommonOptions
 open CommonTypes
+open Gettext
+
+let _s x = _s "CommonWeb" x
+let _b x = _b "CommonWeb" x
+
+let log_prefix = "[cWeb]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
+let lprintf_n fmt =
+  lprintf2 log_prefix fmt
 
 let days = ref 0
 let hours = ref 0
@@ -45,7 +57,8 @@ let add_web_kind kind descr f =
   let kind_record = { f = f; description = descr } in
   file_kinds := (kind, kind_record) :: !file_kinds
 
-let mldonkey_wget url f =
+  
+let mldonkey_wget_url url f =
   let module H = Http_client in
   let r = {
       H.basic_request with
@@ -88,41 +101,48 @@ let mldonkey_wget url f =
     match !date with
       None -> H.wget r f
     | Some date ->
-	let html_time =
-	  begin try
-	    let t = Date.time_of_string date in
-	      r.H.req_save_to_file_time <- t;
-	      Unix.gmtime t
-	  with e ->
-	    let t = Unix.time () in
-	      r.H.req_save_to_file_time <- t;
-	      Unix.gmtime t
-	  end
-	in
 	let file = Filename.concat "web_infos" (Filename.basename r.H.req_url.Url.short_file) in
+	r.H.req_save_to_file_time <- (begin try
+	    Date.time_of_string date
+	  with e ->
+	    Unix.time ()
+	  end);
 	if not (Sys.file_exists file) then
 	  H.wget r f
 	else
 	  begin
 	    let file_date = Unix.LargeFile.stat file in
-	    let file_time = Unix.gmtime file_date.Unix.LargeFile.st_mtime in
-	      if html_time <= file_time then
+	      if r.H.req_save_to_file_time <= file_date.Unix.LargeFile.st_mtime then
 	        begin
-	        lprintf_nl "[cWeb] using local version of %s, HTML header (%s)" file date;
+	        lprintf_nl (_b "using local version of %s (%s), HTML header (%s)")
+		  file (Date.to_full_string file_date.Unix.LargeFile.st_mtime) date;
 	        (f file : unit)
 	        end
 	      else
 	        begin
-	          lprintf_nl "[cWeb] downloading newer %s, HTML header (%s)" file date;
+	          lprintf_nl (_b "downloading newer %s, HTML header (%s)") file date;
 	          H.wget r f
 	        end
 	  end
       )
     with e -> 
-      lprintf_nl "[cWeb] Exception %s while loading %s"
+      lprintf_nl (_b "Exception %s while loading %s")
         (Printexc2.to_string e) url
     end
 
+let mldonkey_wget_shell url f =
+  let command_urlencoded = Str.string_after url 8 in
+  let command = Url.decode command_urlencoded in
+  let filename = Filename.temp_file "wget_" ".tmp" in
+    ignore (Sys.command (Printf.sprintf "%s > %s" command filename));
+    (f filename : unit)
+      
+let mldonkey_wget url f =
+  if Str.string_match (Str.regexp "shell://") url 0 then
+    mldonkey_wget_shell url f
+  else
+    mldonkey_wget_url url f
+      
 let load_url can_fail kind url =
   let f =
     try
@@ -130,21 +150,21 @@ let load_url can_fail kind url =
     with e -> failwith (Printf.sprintf "Unknown kind [%s]" kind)
   in
   try
-    lprintf_nl "[cWeb=%s] saving %s" kind url;
+    lprintf_nl (_b "saving %s (%s)") kind url;
     mldonkey_wget url f
   with e ->
     if can_fail then
       failwith (Printf.sprintf "Exception %s while loading %s"
           (Printexc2.to_string e) url)
     else
-      lprintf_nl "[cWeb] Exception %s while loading %s"
+      lprintf_nl (_b "Exception %s while loading %s")
           (Printexc2.to_string e) url
 
 let load_file kind file =
   try
     (List.assoc kind !file_kinds).f file file
   with e ->
-      lprintf_nl "[cWeb] Exception %s while loading kind %s"
+      lprintf_nl (_b "Exception %s while loading kind %s")
         (Printexc2.to_string e) kind
 
 (*************************************************************************)
@@ -160,7 +180,7 @@ let load_web_infos core_start force =
         try
           load_url false kind url
 	with e ->
-            lprintf_nl "[cWeb] %s while loading %s"
+            lprintf_nl (_b "%s while loading %s")
 	      (Printexc2.to_string e) url
       end
   ) !!CommonOptions.web_infos
@@ -172,13 +192,47 @@ type rss_feed = {
 
 let rss_feeds = Hashtbl.create 10
 
-
 let _ =
   add_web_kind "rss" "Syndication feeds to get periodically updated data" 
     (fun url filename ->
-      lprintf_nl "[cWeb=rss] parsing feed %s" url;
-      let c = Rss.channel_of_file filename in
-      (try Sys.remove filename with _ -> ());
+      lprintf_nl (_b "parsing feed %s (rss)") url;
+      let c =
+	(try
+	  let rss_c = Rss.channel_of_file filename in
+	  (try Sys.remove filename with _ -> ());
+	  rss_c
+	with Xml.Error _ ->
+	  lprintf_nl (_b "found buggy feed, preprocessing with %s and trying again") !!rss_preprocessor;
+	  (try
+	    let pipe_out, pipe_in = Unix.pipe () in
+	    let pid = Unix.create_process !!rss_preprocessor [| !!rss_preprocessor; filename |] 
+		Unix.stdin pipe_in pipe_in in
+	    Unix.close pipe_in;
+	    let output = Buffer.create 1024 in
+	    let buffersize = 1024 in
+	    let buffer = String.create buffersize in
+	    (try
+	      while true do
+		let nread = Unix.read pipe_out buffer 0 buffersize in
+		if nread = 0 then raise End_of_file;
+		Buffer.add_substring output buffer 0 nread
+	      done
+	     with 
+	     | End_of_file -> ()
+	     | Unix.Unix_error (code, f, arg) ->
+		 lprintf_nl "%s failed: %s" !!rss_preprocessor (Unix.error_message code));
+	    (try Unix.close pipe_out with _ -> ());
+	    (try Sys.remove filename with _ -> ());
+	    let _pid, _ = Unix.waitpid [] pid in
+	    let result = Buffer.contents output in
+	    if result = "" then begin
+	      lprintf_nl (_b "%s produced empty content for feed %s, program missing?") !!rss_preprocessor url;
+	      raise Not_found
+	    end;
+	    Rss.channel_of_string result
+	  with Unix.Unix_error (code, f, arg) ->
+	    lprintf_nl (_b "%s failed: %s") !!rss_preprocessor (Unix.error_message code); raise Not_found))
+      in
       let feed =
         try Hashtbl.find rss_feeds url with
           Not_found ->
