@@ -21,6 +21,8 @@ open Md4
 
 open Printf2
 open CommonClient
+open CommonShared
+open CommonServer
 open CommonNetwork
 open CommonResult
 open CommonFile
@@ -49,19 +51,11 @@ let lprintf_n fmt =
   lprintf2 log_prefix fmt
 
 let verify_user_admin () =
-  let empty_pwd = ref false in
-  begin try
-    if user2_password admin_user = blank_password then
-      empty_pwd := true
-    with e ->
-      lprintf_nl "SECURITY INFO: user 'admin' has to be present, creating...";
-      empty_pwd := true;
-      ignore (user2_add admin_user blank_password "")
-  end;
   let warning =
     "SECURITY WARNING: user admin has an empty password, use command: useradd admin password\n"
   in
-  if !empty_pwd && not !!enable_user_config then
+  if has_empty_password admin_user && !!allowed_ips <>
+    [(Ip.range_of_string (strings_of_option allowed_ips).option_default)] then
     begin
       lprintf_n "%s" warning;
       warning
@@ -82,9 +76,16 @@ let dns_works = ref false
 let real_startup_message () =
   let s =
   !startup_message ^ (verify_user_admin ()) ^ (check_supported_os ()) 
-  ^ (if not !dns_works then "DNS resolution does not work" else "")
+  ^ (if not !dns_works then "DNS resolution does not work\n" else "")
+  ^ (if not !Charset.conversion_enabled then "Charset conversion does not work, disabled\n" else "")
+  ^ (match !created_new_base_directory with
+       None -> ""
+     | Some dir -> (Printf.sprintf "MLDonkey created a new home directory in %s\n" dir))
+  ^ (if not !allow_saving_ini_files then "Base directory is full, ini file saving disabled\n" else "")
+  ^ (if !all_temp_queued then "Temp directory is full, all downloads are queued\n" else "")
+  ^ (if !hdd_full_log_closed then "Logfile directory is full, logging redirected to RAM\n" else "")
   ^ (if Autoconf.donkey = "yes" && not !!enable_servers && !!enable_donkey then
-     "You disabled option enable_servers, you will not be able to connect to ED2K servers"
+     "You disabled option enable_servers, you will not be able to connect to ED2K servers\n"
      else "")
   in
   if s = "" then None else Some s
@@ -93,42 +94,68 @@ let hdd_check () =
   let dir_full dir mb =
     match Unix32.diskfree dir with
     | None -> false
+    | Some v when ((Unix32.filesystem dir = "NFS_SUPER_MAGIC") && v = zero) -> false
     | Some v -> v < megabytes mb
   in
 
   if dir_full !!temp_directory !!hdd_temp_minfree then
     if !!hdd_temp_stop_core then begin
-      send_dirfull_warning !!temp_directory "MLDonkey temp directory partition full, shutting down...";
+      send_dirfull_warning !!temp_directory true "MLDonkey shuts down";
       CommonInteractive.clean_exit 0
       end
     else begin
-      send_dirfull_warning !!temp_directory "MLDonkey queues all downloads";
+      send_dirfull_warning !!temp_directory true "MLDonkey queues all downloads";
       all_temp_queued := true
     end
   else
     begin
       all_temp_queued := false;
-      try Hashtbl.remove last_sent_dir_warning !!temp_directory with _ -> ()
+      try
+	ignore (Hashtbl.find last_sent_dir_warning !!temp_directory);
+        (try Hashtbl.remove last_sent_dir_warning !!temp_directory with Not_found -> ());
+        send_dirfull_warning !!temp_directory false "MLDonkey unqueues all downloads"
+      with Not_found -> ()
     end;
 
   let core_dir = Sys.getcwd () in
   if dir_full core_dir !!hdd_coredir_minfree then
     if !!hdd_coredir_stop_core then begin
-      send_dirfull_warning core_dir "MLDonkey base directory partition full, shutting down...";
+      send_dirfull_warning core_dir true "MLDonkey shuts down";
       CommonInteractive.clean_exit 0
       end
     else
       begin
-        send_dirfull_warning core_dir "MLDonkey base directory partition full, stop saving ini files...";
-        allow_saving_ini_files := false
+        allow_saving_ini_files := false;
+        send_dirfull_warning core_dir true "MLDonkey base directory partition full, stop saving ini files"
       end
   else
-    allow_saving_ini_files := true;
+    begin
+      allow_saving_ini_files := true;
+      try
+	ignore (Hashtbl.find last_sent_dir_warning core_dir);
+        (try Hashtbl.remove last_sent_dir_warning core_dir with Not_found -> ());
+        send_dirfull_warning core_dir false "MLDonkey base directory partition has enough free space again, saving ini files again"
+      with Not_found -> ()
+    end;
 
+  if !!log_file <> "" && (not (keep_console_output ())) then begin
   let log_dir = Filename.dirname !!log_file in
-  if dir_full log_dir !!hdd_coredir_minfree then begin
-    send_dirfull_warning log_dir "MLDonkey logdirectory partition full, redirect log to RAM...";
-    close_log ()
+  if dir_full log_dir !!hdd_coredir_minfree then
+    begin
+      hdd_full_log_closed := true;
+      send_dirfull_warning log_dir true "MLDonkey logdirectory partition full, redirect log to RAM";
+      close_log ()
+    end
+  else
+    begin
+      if !hdd_full_log_closed then log_file =:= !!log_file;
+      hdd_full_log_closed := false;
+      try
+	ignore (Hashtbl.find last_sent_dir_warning log_dir);
+        (try Hashtbl.remove last_sent_dir_warning log_dir with Not_found -> ());
+        send_dirfull_warning log_dir false "MLDonkey logdirectory partition has enough free space again, re-enabling logging"
+      with Not_found -> ()
+    end
   end
 
 (* ripped from gui_downloads *)
@@ -207,6 +234,8 @@ if !!html_mods_use_relative_availability
 	then file_availability f
 	else string_availability f.file_availability
 
+let number_of_comments f =
+  List.length f.file_comments
 
 (* WARNING: these computations are much more expensive as they seem.
 We use the ShortLazy to avoid recomputing the result too many times,
@@ -315,7 +344,7 @@ type table_align =
 
 let col_sep = " "
 let add buf s align max_len =
-  let slen = Charset.utf8_length s in
+  let slen = try Charset.utf8_length s with e -> String.length s in
   let diff = max_len - slen in
   match align with
     Align_Center ->
@@ -339,7 +368,7 @@ let print_table_text buf alignments titles lines =
   List.iter (fun line ->
       let len = Array.length line in
       for i = 0 to len-1 do
-        let slen = Charset.utf8_length line.(i) in
+        let slen = try Charset.utf8_length line.(i) with e -> String.length line.(i) in
         if cols.(i) <  slen then cols.(i) <- slen
       done;
   ) (titles :: lines);
@@ -543,7 +572,7 @@ function cancelAll(x){for(i=0;i\\<document.selectForm.elements.length;i++){var j
   Printf.bprintf buf "\\</form\\>"
 
 
-  let print_file_html_mods buf guifiles =
+let print_file_html_mods buf guifiles =
 
     if (List.length guifiles) > 0 then begin
 	let tsize = ref Int64.zero in
@@ -604,7 +633,7 @@ function submitPriority(num,cp,sel) {
 \\<div class=main\\>";
 
 if !!html_mods_use_js_tooltips then Printf.bprintf buf 
-"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:FFFFDD;color:black;border-color:black;border-width:20;font-size:8pt; visibility:show; left:25px; top:
+"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:
 -100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
 
 	Printf.bprintf buf "\\<form id=\\\"selectForm\\\" name=\\\"selectForm\\\" action=\\\"files\\\"\\>
@@ -628,7 +657,8 @@ if !!html_mods_use_js_tooltips then Printf.bprintf buf
 
 \\<td title=\\\"Pause\\\"  class=\\\"dlheader np\\\"\\>P\\</td\\>
 \\<td title=\\\"Resume\\\" class=\\\"dlheader np\\\"\\>R\\</td\\>
-\\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>"
+\\<td title=\\\"Cancel\\\" class=\\\"dlheader brs\\\"\\>C\\</td\\>
+\\<td title=\\\"Click to switch release status\\\" class=\\\"dlheader brs\\\"\\>R\\</td\\>"
 (if !qnum > 0 then begin
 	Printf.sprintf "title=\\\"Active(%d): %s/%s | Queued(%d): %s/%s\\\""
 	(List.length guifiles - !qnum) (size_of_int64 (!tdl -- !qdl)) (size_of_int64 (!tsize -- !qsize))
@@ -638,17 +668,29 @@ else "")
 (List.length guifiles) (size_of_int64 !tdl) (size_of_int64 !tsize) (!trate /. 1024.)
 (let unread = ref 0 in
 Fifo.iter (fun (t,i,num,n,s) -> if t > !last_message_log then incr unread) chat_message_fifo;
-if !unread > 0 then Printf.sprintf "\\<td class=downloaded title=\\\"%d unread messages\\\"\\>(+%d)\\&nbsp;\\</td\\>" !unread !unread else "");
+if !unread > 0 then Printf.sprintf "\\<td class=downloaded title=\\\"%d unread messages\\\"\\>\\<a onClick=\\\"mSub('fstatus','version');mSub('output','message')\\\"\\>(+%d)\\</a\\>\\&nbsp;\\</td\\>" !unread !unread else "");
 
 if !!html_mods_vd_network then Printf.bprintf buf
 "\\<td title=\\\"Sort by network\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=N name=sortby\\>\\</td\\>";
 
 Printf.bprintf buf
-"\\<td title=\\\"Sort by filename\\\" class=dlheader\\>\\<input class=headbutton type=submit value=File name=sortby\\>\\</td\\>
-\\<td title=\\\"Sort by size\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Size name=sortby\\>\\</td\\>
+"\\<td title=\\\"Sort by filename\\\" class=dlheader\\>\\<input class=headbutton type=submit value=File name=sortby\\>\\</td\\>";
+
+if !!html_mods_vd_user then Printf.bprintf buf
+"\\<td title=\\\"Sort by user\\\" class=dlheader\\>\\<input class=headbutton type=submit value=User name=sortby\\>\\</td\\>";
+
+if !!html_mods_vd_group then Printf.bprintf buf
+"\\<td title=\\\"Sort by group\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Group name=sortby\\>\\</td\\>";
+
+Printf.bprintf buf
+"\\<td title=\\\"Sort by size\\\" class=dlheader\\>\\<input class=headbutton type=submit value=Size name=sortby\\>\\</td\\>
 \\<td title=\\\"Sort by size downloaded\\\" class=dlheader\\>\\<input class=\\\"headbutton ar\\\" type=submit value=DLed name=sortby\\>\\</td\\>
-\\<td title=\\\"Sort by percent\\\" class=dlheader\\>\\<input class=headbutton type=submit value=%% name=sortby\\>\\</td\\>
-\\<td title=\\\"Sort by number of sources\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=Srcs name=sortby\\>\\</td\\>";
+\\<td title=\\\"Sort by percent\\\" class=dlheader\\>\\<input class=headbutton type=submit value=%% name=sortby\\>\\</td\\>";
+if !!html_mods_vd_comments then Printf.bprintf buf
+"\\<td title=\\\"Sort by comments\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=Cm name=sortby\\>\\</td\\>";
+
+Printf.bprintf buf
+"\\<td title=\\\"Sort by number of sources\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=Srcs name=sortby\\>\\</td\\>";
 
 if !!html_mods_vd_active_sources then Printf.bprintf buf
 "\\<td title=\\\"Sort by number of active sources\\\" class=dlheader\\>\\<input style=\\\"padding-left: 0px; padding-right: 0px;\\\" class=headbutton type=submit value=A name=sortby\\>\\</td\\>";
@@ -678,18 +720,36 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
         [|
           (if !!html_mods_use_js_tooltips then
                         Printf.sprintf "
-                                onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%sFile#: %d<br>Network: %s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
-                        (Http_server.html_real_escaped file.file_name)
+                                onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%sFile#: %d<br>Network: %s<br>User%s %s%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
+                        (Http_server.html_real_escaped (Charset.to_utf8 file.file_name))
 			(match file_magic (file_find file.file_num) with
 			   None -> ""
 			 | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
 			file.file_num
                         (net_name file)
+                        (if file.file_group = "none" then "" else ":Group")
+                        file.file_user
+                        (if file.file_group = "none" then "" else Printf.sprintf ":%s" file.file_group)
+
+      (if file.file_comments = [] then "" else
+			    begin
+           let num_comments = number_of_comments file in
+           let buf1 = Buffer.create (!!max_comment_length * num_comments) in
+           Printf.bprintf buf1 "<br><br>Comments(%d):<br>" (num_comments);
+	   let comments =
+	     if List.length file.file_comments > 5 then
+	       fst (List2.cut 5 file.file_comments) @ [Ip.null, "", 0, (_s "MLDonkey note: click file for more comments")]
+	     else
+	       file.file_comments
+	   in
+           List.iter (fun (_,_,_,s) -> Printf.bprintf buf1 "%s<br>" (Http_server.html_real_escaped (Charset.to_utf8 s))) comments;
+			      Buffer.contents buf1
+			    end)
+
 			!!html_mods_js_tooltips_wait
 			!!html_mods_js_tooltips_timeout
 			!!html_mods_js_tooltips_wait
-			 else Printf.sprintf "
-                               onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
+       else Printf.sprintf " onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
 
 	(if downloading file then
               Printf.sprintf "\\<td class=\\\"dl al np\\\"\\>\\<input class=checkbox name=pause type=checkbox value=%d\\>\\</td\\>
@@ -704,6 +764,10 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
                 file.file_num
                 file.file_num);
 
+	  Printf.sprintf "\\<td onClick=\\\"location.href='files?%s=%d';return true;\\\" class=\\\"dl al brs\\\"\\>\\%s\\</td\\>"
+	    (if file.file_release then "norelease" else "release")
+	    file.file_num (if file.file_release then "R" else "-");
+
           (if !!html_mods_vd_network then
 			Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d';return true;\\\"
 			title=\\\"%s\\\" class=\\\"dl al\\\"\\>%s\\</td\\>"
@@ -727,14 +791,14 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
             (truncate ( (1. -. downloaded /. size) *. 100.))
 	 else
 	     Printf.sprintf "\\<TD onClick=\\\"location.href='submit?q=vd+%d';return true;\\\"
-                        title=\\\"[File#: %d] [Net: %s]%s\\\" class=\\\"dl al\\\"\\>%s\\<br\\>
+                        title=\\\"[File#: %d] [Net: %s] [Comments: %d]%s\\\" class=\\\"dl al\\\"\\>%s\\<br\\>
                         \\<table cellpadding=0 cellspacing=0 width=100%%\\>\\<tr\\>
                         \\<td class=\\\"loaded\\\" style=\\\"height:%dpx\\\" width=\\\"%d%%\\\"\\> \\</td\\>
                         \\<td class=\\\"remain\\\" style=\\\"height:%dpx\\\" width=\\\"%d%%\\\"\\> \\</td\\>
                         \\</tr\\>\\</table\\>\\</td\\>"
             file.file_num
             file.file_num
-                        (net_name file)
+                        (net_name file) (number_of_comments file)
                         (if !!max_name_len < String.length file.file_name then " " ^ file.file_name else "")
             (short_name file)
             (!!html_vd_barheight)
@@ -743,9 +807,17 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
             (truncate ( (1. -. downloaded /. size) *. 100.)));
           );
 
+          (if !!html_mods_vd_user then ctd file.file_num file.file_user else "");
+          (if !!html_mods_vd_group then ctd file.file_num file.file_group else "");
+
           (ctd file.file_num (size_of_int64 file.file_size));
           (ctd file.file_num (size_of_int64 file.file_downloaded));
           (ctd file.file_num (Printf.sprintf "%.1f" (percent file)));
+
+          (if !!html_mods_vd_comments then
+            ctd file.file_num (Printf.sprintf "%d" (number_of_comments file))
+            else "");
+
           (ctd file.file_num (Printf.sprintf "%d" (number_of_sources file)));
 
           (if !!html_mods_vd_active_sources then
@@ -808,7 +880,7 @@ let ctd fn td = Printf.sprintf "\\<td onClick=\\\"location.href='submit?q=vd+%d'
 end
 else
   html_mods_table_one_row buf "downloaderTable" "downloaders" [
-    ("", "srh", "!! No files, please use search or the dllink <url> command to add a new download !!"); ]
+    ("", "srh", (Printf.sprintf (_b "!! No files, please use search or the dllink <url> command to add a new download !!"))); ]
 
 let html_mods_done_files buf files =
 
@@ -855,11 +927,11 @@ let html_mods_done_files buf files =
   Printf.bprintf buf "\\</form\\>"
 
 let print_human_readable file size =
- (if Int64.to_float size > 1024. && Int64.to_float size < 1048576. then
+ (if Int64.to_float size >= 1024. && Int64.to_float size < 1048576. then
     (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1024.) ("kb") )
-  else if size > Int64.of_float 1048576. && Int64.to_float size < 1073741824. then
+  else if size >= Int64.of_float 1048576. && Int64.to_float size < 1073741824. then
     (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1048576.) ("mb") )
-  else if size > Int64.of_float 1073741824. then
+  else if size >= Int64.of_float 1073741824. then
     (Printf.sprintf "%5.1f%s" (Int64.to_float size /. 1073741824.) ("gb") )
   else if size < Int64.zero then
     (Printf.sprintf "%d chunks"
@@ -876,7 +948,7 @@ let simple_print_file_list finished buf files format =
   let print_table = if format.conn_output = HTML then print_table_html 2
     else print_table_text in
   if not finished then
-    if format.conn_output = HTML && ( !!html_checkbox_vd_file_list  ) then
+    if format.conn_output = HTML && !!html_checkbox_vd_file_list then
       begin
         if !!html_mods then print_file_html_mods buf files
         else
@@ -885,7 +957,7 @@ let simple_print_file_list finished buf files format =
     else
       print_table buf
         [|
-        Align_Left; Align_Left; Align_Right; Align_Right;
+        Align_Left; Align_Left; Align_Right; Align_Left; Align_Left; Align_Left;
         Align_Right; Align_Right; Align_Right |]
         (if format.conn_output = HTML then
           [|
@@ -900,6 +972,10 @@ let simple_print_file_list finished buf files format =
           |] else
           [|
             "$nNum";
+	    "Rele";
+	    "Comm";
+	    "User";
+	    "Group";
             "File";
             "    %";
             "    Done";
@@ -944,6 +1020,10 @@ let simple_print_file_list finished buf files format =
                     file.file_num
                       (if downloading file then "PAUSE" else "RESUME")
                   else ""));
+	      (Printf.sprintf "%s" (if file.file_release then "R" else "-"));
+	      (Printf.sprintf "%4d" (number_of_comments file));
+              file.file_user;
+              file.file_group;
               (short_name file);
               (Printf.sprintf "%3.1f" (percent file));
               (if !!improved_telnet then (print_human_readable file file.file_downloaded)
@@ -1061,6 +1141,9 @@ let display_active_file_list buf o list =
         | ByLast -> (fun f1 f2 -> f1.file_last_seen >= f2.file_last_seen)
         | ByNet -> (fun f1 f2 -> net_name f1 <= net_name f2)
         | ByAvail -> (fun f1 f2 -> get_file_availability f1 >= get_file_availability f2)
+        | ByComments -> (fun f1 f2 -> (number_of_comments f1) >= (number_of_comments f2))
+        | ByUser -> (fun f1 f2 -> f1.file_user <= f2.file_user)
+        | ByGroup -> (fun f1 f2 -> f1.file_group <= f2.file_group)
         | NotSorted -> raise Not_found
       in
       Sort.list sorter list
@@ -1098,15 +1181,17 @@ let get_tag_value tag =
   match tag.tag_value with
  | Uint64 i -> String.escaped (Int64.to_string i)
  | Fint64 i -> String.escaped (Int64.to_string i)
+ | Uint16 i | Uint8 i -> String.escaped (string_of_int i)
  | String s -> String.escaped s
- | _ -> ""
+ | Addr i -> Ip.to_string i
+ | Pair (x,y) -> Printf.sprintf "%Ld, %Ld" x y
 
 let old_print_search buf o results =
   let user = o.conn_user in
   let counter = ref 0 in
   if use_html_mods o then
   begin
-    if !!html_mods_use_js_tooltips then Printf.bprintf buf "\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:FFFFDD;color:black;border-color:black;border-width:20;font-size:8pt; visibility:show; left:25px; top:-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>\n";
+    if !!html_mods_use_js_tooltips then Printf.bprintf buf "\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>\n";
     html_mods_table_header_colspan buf "resultsTable" "results" [
       ( "1", "0", "srh", "Network", "Network" ) ;
       ( "1", "0", "srh", "File", "File (mouseover)" ) ;
@@ -1140,7 +1225,19 @@ let old_print_search buf o results =
               in
               
                 if use_html_mods o 
-                  then Printf.bprintf buf "\\<td class=\\\"sr\\\"\\>%s\\</td\\>" network_name
+                  then Printf.bprintf buf "\\<td class=\\\"sr\\\"\\>%s\\</td\\>"
+		   (let rec iter uids =
+		      match uids with
+			[] -> network_name
+		      | uid :: tail ->
+			  match Uid.to_uid uid with
+			    Ed2k md4 ->
+			      Printf.sprintf "\\<a href=\\\"%s\\\"\\>%s\\</a\\>"
+				(file_print_ed2k_link (List.hd r.result_names) r.result_size md4)
+				network_name
+			    | _  -> iter tail
+		    in
+		    iter r.result_uids)
                   else Printf.bprintf  buf "[%5d] %s " !counter network_name;
 
               if o.conn_output = HTML then begin
@@ -1639,7 +1736,8 @@ let networks_header buf =
       ( "0", "srh br", "Has search", "Search" ) ;
       ( "0", "srh br", "Has chat", "Chat" ) ;
       ( "0", "srh br", "Has rooms", "Rooms" ) ;
-      ( "0", "srh", "Has multinet", "Multinet" ) ]
+      ( "0", "srh", "Has multinet", "Multinet" ) ;
+      ( "0", "srh", "Has porttest", "Porttest" ) ]
 
 let print_network_modules buf o =
   let buf = o.conn_buf in
@@ -1654,17 +1752,22 @@ let print_network_modules buf o =
           if not (List.mem VirtualNetwork n.network_flags) then
             try
               let net_has e = if List.mem e n.network_flags then "yes" else "" in
+              let net_has_porttest () =
+		match network_porttest_result n with
+		  PorttestNotAvailable -> ""
+		| _ -> "yes" in
               Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
                 ("", "sr br", n.network_name);
-                ("", "sr br", if n.op_network_is_enabled () then "Enabled" else "Disabled");
+                ("", "sr br", if network_is_enabled n then "Enabled" else "Disabled");
                 ("", "sr br", net_has NetworkHasUpload);
                 ("", "sr br", net_has NetworkHasServers);
                 ("", "sr br", net_has NetworkHasSupernodes);
                 ("", "sr br", net_has NetworkHasSearch);
                 ("", "sr br", net_has NetworkHasChat);
                 ("", "sr br", net_has NetworkHasRooms);
-                ("", "sr"   , net_has NetworkHasMultinet); ];
+                ("", "sr br", net_has NetworkHasMultinet);
+                ("", "sr"   , (net_has_porttest ())); ];
               Printf.bprintf buf "\\</tr\\>";
             with _ -> ()
         );
@@ -1679,13 +1782,11 @@ let print_network_modules buf o =
   else
     begin
       Printf.bprintf buf "Networks:";
-      Hashtbl.iter
-        (fun name n ->
+      networks_iter_all (fun n ->
           try
-            Printf.bprintf buf "\n   %2d %-30s %s" n.network_num name
-                (if n.op_network_is_enabled () then "Enabled" else "Disabled")
-          with _ -> ()
-        ) networks_by_name
+            Printf.bprintf buf "\n   %2d %-30s %s" n.network_num n.network_name
+                (if network_is_enabled n then "Enabled" else "Disabled")
+          with _ -> ())
     end
 
 let print_gdstats buf o =
@@ -1764,7 +1865,7 @@ let buildinfo html buf =
   tack list
     (
       "Networks:\t", 
-      !networks_string 
+      !networks_string
     );
   tack list 
     (
@@ -1803,7 +1904,7 @@ let buildinfo html buf =
   tack list 
     (
       "Features:\t",
-      (if BasicSocket.has_threads () then "threads" else "no-threads") ^
+      (if BasicSocket.has_threads () then " threads" else " no-threads") ^
       (let s = Zlib.zlib_version_num () in 
          Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else "")) ^
       (if Autoconf.bzip2 then
@@ -1822,7 +1923,10 @@ let buildinfo html buf =
 	    " gd(jpg)"
 	| _, false, false ->
 	    " gd(neither jpg nor png ?)") ^
-      (if Autoconf.has_iconv then " iconv" else " no-iconv") ^
+      (match Autoconf.has_iconv, !Charset.conversion_enabled with
+       | true, true -> " iconv(active)"
+       | true, false -> " iconv(inactive)"
+       | false, _ -> " no-iconv") ^
       (match Autoconf.magic, !Autoconf.magic_works with
        | true, true -> " magic(active)"
        | true, false -> " magic(inactive)"
@@ -1860,8 +1964,8 @@ let runinfo html buf o =
     (
       "User:\t\t", 
       Printf.sprintf "%s (%s) - uptime: %s" 
-	o.conn_user.ui_user_name
-	(if empty_password o.conn_user.ui_user_name then "Warning: empty Password"
+	o.conn_user.ui_user.user_name
+	(if has_empty_password o.conn_user.ui_user then "Warning: empty Password"
 	else "PW Protected")
 	(Date.time_to_string (last_time () - start_time) "verbose")
     );
@@ -1967,7 +2071,51 @@ let runinfo html buf o =
     ) list;
     if html then 
       Printf.bprintf buf "\\</table\\>\\</div\\>"
-          
+
+type port_info = {
+    netname : string;
+    port : int;
+    portname : string
+  }
+
+let portinfo html buf =
+  let network_name_list_width = ref 7 in (* "Network" *)
+  let list = ref [] in
+  networks_iter (fun r ->
+    if String.length r.network_name > !network_name_list_width then
+      network_name_list_width := String.length r.network_name;
+    List.iter (fun (p,s) -> if p <> 0 then list := !list @
+      [{netname = r.network_name; port = p; portname = s}]) (network_ports r)
+  );
+  List.iter (fun (p,s) -> if p <> 0 then list := !list @
+      [{netname = "Core"; port = p; portname = s}])
+    (network_ports (network_find_by_name "Global Shares"));
+
+  let fill_network s = String.make (max 0 (!network_name_list_width - 7)) s in
+  if html then
+      html_mods_table_header buf "sharesTable" "shares" [
+       ( "0", "srh", "Network", "Network" ) ;
+       ( "0", "srh ar", "Port", "Port" ) ;
+       ( "1", "srh", "Type", "Type" ) ]
+  else
+    begin
+      Printf.bprintf buf "\n\t--Portinfo--\n";
+      Printf.bprintf buf "Network%s|  Port|Type\n" (fill_network ' ');
+      Printf.bprintf buf "-------%s+------+-------------------\n" (fill_network '-')
+    end;
+
+  html_mods_cntr_init ();
+  List.iter (fun p ->
+    if html then
+      Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>\\<td class=\\\"sr ar\\\"\\>%d\\</td\\>\\<td class=\\\"sr\\\"\\>%s\\</td\\>"
+	(html_mods_cntr ()) p.netname p.port p.portname
+    else
+      Printf.bprintf buf "%-*s|%6d|%s\n"
+	(max !network_name_list_width (!network_name_list_width - String.length p.netname)) p.netname p.port p.portname
+    ) (List.sort (fun p1 p2 -> String.compare p1.netname p2.netname) !list);
+  if html then
+    Printf.bprintf buf "\\</table\\>\\</td\\>\\<tr\\>\\</table\\>\\</div\\>"
+
 let diskinfo html buf =
   let list = ref [] in
   ignore (search_incoming_files ());
@@ -1981,8 +2129,8 @@ let diskinfo html buf =
   let len_dir = ref 9 in
   let len_strategy = ref 29 in (* "shared (incoming_directories)" *)
   List.iter ( fun (dir, strategy) ->
-    len_dir := maxi !len_dir (String.length dir);
-    len_strategy := maxi !len_strategy (String.length strategy)
+    len_dir := max !len_dir (String.length dir);
+    len_strategy := max !len_strategy (String.length strategy)
   ) !list;
   let fill_dir = String.make (!len_dir - 9) ' ' in
   let fill_dir_line = String.make (!len_dir - 9) '-' in
@@ -2031,8 +2179,8 @@ let diskinfo html buf =
 	  end
 	else
 	  Printf.bprintf buf "%-*s|%-*s|%8s|%8s|%5s|%-s\n"
-	    (maxi !len_dir (!len_dir - String.length dir)) dir
-	    (maxi !len_strategy (!len_strategy - String.length strategy)) strategy
+	    (max !len_dir (!len_dir - String.length dir)) dir
+	    (max !len_strategy (!len_strategy - String.length strategy)) strategy
 	    diskused diskfree percentfree filesystem
     	) !list;
   if html then
@@ -2078,11 +2226,11 @@ let dllink_print_result html url header results =
   if html then Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>\\</div\\>";
   Buffer.contents buf
 
-let dllink_query_networks html url =
+let dllink_query_networks html url user =
   let result = ref [] in
   if not (networks_iter_until_true (fun n ->
     try
-      let s,r = network_parse_url n url in
+      let s,r = network_parse_url n url user in
         if s = "" then
           r
         else
@@ -2100,7 +2248,7 @@ let dllink_query_networks html url =
   else
     dllink_print_result html url "Added link" !result
 
-let dllink_parse html url =
+let dllink_parse html url user =
   if (String2.starts_with url "http") then (
     let u = Url.of_string url in
     let module H = Http_client in
@@ -2134,11 +2282,248 @@ let dllink_parse html url =
       let concat_headers =
         (List.fold_right (fun (n, c) t -> n ^ ": " ^ c ^ "\n" ^ t) headers "")
       in
-      ignore (dllink_query_networks html concat_headers)
+      ignore (dllink_query_networks html concat_headers user)
     );
     dllink_print_result html url "Parsing HTTP url" [])
   else
     if (String2.starts_with url "ftp") then
-      dllink_query_networks html (Printf.sprintf "Location: %s" url)
+      dllink_query_networks html (Printf.sprintf "Location: %s" url) user
     else
-      dllink_query_networks html url
+      dllink_query_networks html url user
+
+module UnionFind = struct
+  type t = int array
+  let create_sets n =
+    Array.init n (fun i -> i) (* each element is its own leader *)
+  let find_leader t i =
+    let rec fix_point i =
+      let parent = t.(i) in
+      if parent <> i then fix_point parent 
+      else i in
+    let leader = fix_point i in
+    t.(i) <- leader;
+    leader
+  let merge_sets t i j =
+    let leaderi = find_leader t i in
+    let leaderj = find_leader t j in
+    t.(leaderi) <- leaderj
+  let number_of_sets t =
+    let nsets = ref 0 in
+    Array.iteri (fun i ti ->
+      if i = ti then incr nsets) t;
+    !nsets
+end
+
+let filenames_variability o list =
+  (* over this number of filenames, exact variability is not computed
+     (too expensive) *)
+  let bypass_threshold = 100 in
+  (* minimum distance that must exist between two groups of filenames
+     so they're considered separate *)
+  let gap_threshold = 4 in
+
+  let buf = o.conn_buf in
+
+  let is_alphanum = function
+    | 'A' .. 'Z'
+    | 'a' .. 'z'
+    | '0' .. '9' -> true
+    | _ -> false in
+
+  let canonized_words s =
+    let len = String.length s in
+    let current_word = Buffer.create len in
+    let rec outside_word i wl =
+      if i < len then
+	if not (is_alphanum s.[i]) then outside_word (i + 1) wl
+	else begin (* start of a new word *)
+	  Buffer.add_char current_word (Char.lowercase s.[i]);
+	  inside_word (i + 1) wl
+	end
+      else wl
+    and inside_word i wl =
+      if i < len then 
+	if not (is_alphanum s.[i]) then begin (* end of the word *)
+	  let wl = Buffer.contents current_word :: wl in
+	  Buffer.reset current_word;
+	  outside_word i wl
+	end else begin
+	  Buffer.add_char current_word (Char.lowercase s.[i]);
+	  inside_word (i + 1) wl
+	end
+      else Buffer.contents current_word :: wl
+    in
+    outside_word 0 [] in
+
+  let costs = { 
+    Levenshtein.insert_cost = 1; 
+    Levenshtein.delete_cost = 1; 
+    Levenshtein.replace_cost = 2 } in
+  (* we can only assume the distance is symetric if insert and
+     delete costs are the same *)
+  assert (costs.Levenshtein.insert_cost = costs.Levenshtein.delete_cost);
+  let dist = Levenshtein.ForWords.distance costs in
+
+  let score_list =
+    List.map (fun fileinfo ->
+      (* canonize filenames by keeping only lowercase words, and
+	 sorting them so that initial order doesn't matter;
+	 Remove duplicate canonized filenames *)
+      let fns = Array.of_list (List.fold_left (fun acc fn ->
+	let new_fn =
+	  Array.of_list (List.sort String.compare (canonized_words fn)) in
+	if List.mem new_fn acc then acc else new_fn :: acc
+      ) [] fileinfo.file_names) in
+
+      let nfilenames = Array.length fns in
+      if nfilenames > bypass_threshold then
+	fileinfo, bypass_threshold
+      else
+	let unionfind_sets = UnionFind.create_sets nfilenames in
+	for i = 0 to nfilenames - 2 do
+	  let d1 = dist fns.(i) in
+	  for j = i + 1 to nfilenames - 1 do
+	    if d1 fns.(j) < gap_threshold then
+	      UnionFind.merge_sets unionfind_sets i j
+	  done
+	done;
+	fileinfo, UnionFind.number_of_sets unionfind_sets
+    ) list in
+
+  (* files with most clusters at the end of results table *)
+  let sorted_score_list =
+    List.sort (fun (_, nc1) (_, nc2) -> compare nc1 nc2)
+      score_list in
+
+  let print_table = if o.conn_output = HTML then print_table_html 2
+    else print_table_text in
+  print_table buf
+    [|
+      Align_Left; Align_Left; Align_Right |]
+    [| 
+      "Num";
+      "File";
+      "Clusters" |]
+    (List.map (fun (fileinfo, nc) ->
+      let n = network_find_by_num fileinfo.file_network in
+      [| 
+	Printf.sprintf "[%-s %5d]" n.network_name (fileinfo.file_num);
+	shorten fileinfo.file_name 80;
+	string_of_int nc |]
+    ) sorted_score_list)
+
+let print_upstats o list server =
+  let buf = o.conn_buf in
+  if use_html_mods o then
+    begin
+      if !!html_mods_use_js_tooltips then Printf.bprintf buf 
+"\\<div id=\\\"object1\\\" style=\\\"position:absolute; background-color:#FFFFDD;color:black;border-color:black;border-width:20px;font-size:8pt; visibility:visible; left:25px; top:
+-100px; z-index:+1\\\" onmouseover=\\\"overdiv=1;\\\"  onmouseout=\\\"overdiv=0; setTimeout(\\\'hideLayer()\\\',1000)\\\"\\>\\&nbsp;\\</div\\>";
+
+      Printf.bprintf buf "\\<div class=\\\"upstats\\\"\\>";
+      match server with
+	None ->
+	  html_mods_table_one_row buf "upstatsTable" "upstats" [
+	    ("", "srh", Printf.sprintf "Session: %s uploaded | Shared(%d): %s\n"
+	    (size_of_int64 !upload_counter) !nshared_files (size_of_int64 !nshared_bytes)); ]
+      | Some s -> let info = server_info s in
+	  html_mods_table_one_row buf "upstatsTable" "upstats" [
+	    ("", "srh", Printf.sprintf "%d files shared on %s (%s:%s)"
+              info.G.server_published_files info.G.server_name
+              (Ip.string_of_addr info.G.server_addr)
+              (string_of_int info.G.server_port)); ]
+    end
+  else
+    begin
+      Printf.bprintf buf "Upload statistics:\n";
+      Printf.bprintf buf "Session: %s uploaded | Shared(%d): %s\n"
+	(size_of_int64 !upload_counter) !nshared_files (size_of_int64 !nshared_bytes)
+    end;
+
+  if use_html_mods o then
+    html_mods_table_header buf "upstatsTable" "upstats" [
+      ( "1", "srh", "Total file requests", "Reqs" ) ;
+      ( "1", "srh", "Total bytes sent", "Total" ) ;
+      ( "1", "srh", "Upload Ratio", "UPRatio" ) ;
+      ( "0", "srh", "Preview", "P" ) ;
+      ( "0", "srh", "Filename", "Filename" );
+      ( "0", "srh", "Statistic links", "Stats" );
+      ( "0", "srh", "Published on servers", "Publ" ) ]
+  else
+    begin
+      Printf.bprintf buf " Requests |  Bytes   | Uploaded | File\n";
+      Printf.bprintf buf "----------+----------+----------+----------------------------------------------------\n";
+    end;
+
+  html_mods_cntr_init ();
+  let list = Sort.list (fun f1 f2 ->
+      (f1.impl_shared_requests = f2.impl_shared_requests &&
+       f1.impl_shared_uploaded > f2.impl_shared_uploaded) ||
+      (f1.impl_shared_requests > f2.impl_shared_requests )
+    ) list in
+
+  List.iter (fun impl ->
+    if use_html_mods o then
+      begin
+	let published = List.length impl.impl_shared_servers in
+	let ed2k = file_print_ed2k_link
+	  (Filename.basename impl.impl_shared_codedname)
+	  impl.impl_shared_size impl.impl_shared_id in
+
+	Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"" (html_mods_cntr ());
+	(if !!html_mods_use_js_tooltips then
+	   Printf.bprintf buf " onMouseOver=\\\"mOvr(this);setTimeout('popLayer(\\\\\'%s<br>%s%s\\\\\')',%d);setTimeout('hideLayer()',%d);return true;\\\" onMouseOut=\\\"mOut(this);hideLayer();setTimeout('hideLayer()',%d)\\\"\\>"
+	    (Http_server.html_real_escaped (Filename.basename (Charset.to_utf8 impl.impl_shared_codedname)))
+	    (match impl.impl_shared_magic with
+	       None -> ""
+	     | Some magic -> "File type: " ^ (Http_server.html_real_escaped magic) ^ "<br>")
+	    (if impl.impl_shared_servers = [] then "" else
+	       Printf.sprintf "<br>Published on %d %s<br>%s"
+		published (if published = 1 then "server" else "servers")
+		(let listbuf = Buffer.create 100 in
+		   List.iter (fun s -> let info = server_info s in
+		     Printf.bprintf listbuf "%s (%s:%s%s)<br>"
+			info.server_name
+			(Ip.string_of_addr info.server_addr)
+			(string_of_int info.server_port)
+			(if info.server_realport <> 0
+			   then "(" ^ (string_of_int info.server_realport) ^ ")" else "")
+			) impl.impl_shared_servers;
+		 Buffer.contents listbuf))
+		!!html_mods_js_tooltips_wait
+		!!html_mods_js_tooltips_timeout
+		!!html_mods_js_tooltips_wait
+	 else Printf.bprintf buf " onMouseOver=\\\"mOvr(this);return true;\\\" onMouseOut=\\\"mOut(this);\\\"\\>");
+
+	let uploaded = Int64.to_float impl.impl_shared_uploaded in
+	let size = Int64.to_float impl.impl_shared_size in
+
+	html_mods_td buf [
+	  ("", "sr ar", Printf.sprintf "%d" impl.impl_shared_requests);
+	  ("", "sr ar", size_of_int64 impl.impl_shared_uploaded);
+	  ("", "sr ar", Printf.sprintf "%5.1f" ( if size < 1.0 then 0.0 else (uploaded *. 100.) /. size));
+	  ("", "sr", Printf.sprintf "\\<a href=\\\"preview_upload?q=%d\\\"\\>P\\</a\\>" impl.impl_shared_num);
+	  ("", "sr", (if impl.impl_shared_id = Md4.null then
+		        (Filename.basename impl.impl_shared_codedname)
+		      else
+			Printf.sprintf "\\<a href=\\\"%s\\\"\\>%s\\</a\\>"
+			  ed2k (shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)));
+	  ("", "sr", (if impl.impl_shared_id = Md4.null then "" else
+			Printf.sprintf "\\<a href=\\\"http://tothbenedek.hu/ed2kstats/ed2k?hash=%s\\\"\\>%s\\</a\\>
+\\<a href=\\\"http://ed2k.titanesel.ws/ed2k.php?hash=%s\\\"\\>%s\\</a\\>
+\\<a href=\\\"http://bitzi.com/lookup/ed2k:%s\\\"\\>%s\\</a\\>"
+                      (Md4.to_string impl.impl_shared_id) "T1"
+                      (Md4.to_string impl.impl_shared_id) "T2"
+                      (Md4.to_string impl.impl_shared_id) "B"));
+	  ("", "sr ar", Printf.sprintf "%d" published ) ];
+	Printf.bprintf buf "\\</tr\\>\n";
+      end
+    else
+      Printf.bprintf buf "%9d | %8s | %7s%% | %-50s\n"
+	(impl.impl_shared_requests)
+	(size_of_int64 impl.impl_shared_uploaded)
+	(Printf.sprintf "%3.1f" ((Int64.to_float impl.impl_shared_uploaded *. 100.) /. Int64.to_float impl.impl_shared_size))
+	(shorten (Filename.basename impl.impl_shared_codedname) !!max_name_len)
+  ) list;
+
+  if use_html_mods o then Printf.bprintf buf "\\</table\\>\\</div\\>\\</div\\>"
