@@ -388,7 +388,6 @@ let fW_lb_last_seen  = ref ""
 let fW_lb_file_eta  = ref ""
 let fW_lb_rename  = ref ""
 let fW_lb_file_names_col  = ref ""
-let fW_lb_file_count_col  = ref ""
 let fW_wt_show_source_details  = ref ""
 let fW_lb_source_info  = ref ""
 let fW_lb_files_requested  = ref ""
@@ -434,7 +433,6 @@ let mW_lb_rooms  = ref ""
 let mW_lb_uploads  = ref ""
 let mW_lb_console  = ref ""
 let mW_lb_graph  = ref ""
-let mW_lb_im  = ref ""
 let mW_lb_settings  = ref ""
 let mW_lb_initializing  = ref ""
 let mW_lb_warning = ref ""
@@ -446,7 +444,6 @@ let mW_me_scan_ports  = ref ""
 let mW_me_reconnect_to  = ref ""
 let mW_me_settings  = ref ""
 let mW_me_main_menu  = ref ""
-let mW_me_im  = ref ""
 let mW_me_restore = ref ""
 
 (* Settings Window *)
@@ -454,7 +451,6 @@ let mW_me_restore = ref ""
 let cW_lb_main  = ref ""
 let cW_lb_interfaces  = ref ""
 let cW_lb_tools  = ref ""
-let cW_lb_mlchat  = ref ""
 let cW_lb_other  = ref ""
 let cW_lb_bittorrent  = ref ""
 let cW_lb_direct_connect  = ref ""
@@ -858,7 +854,6 @@ let load_messages () =
   fW_lb_file_eta  :=  _s_ "ETA (inst / average)";
   fW_lb_rename  :=  _s "Rename file";
   fW_lb_file_names_col  :=  _s "File names";
-  fW_lb_file_count_col  :=  _s "Sources";
   fW_wt_show_source_details  :=  _s "Client details";
   fW_lb_source_info  :=  _s "Client information";
   fW_lb_files_requested  :=  _s "Files requested";
@@ -904,7 +899,6 @@ let load_messages () =
   mW_lb_uploads  :=  _s "_Uploads";
   mW_lb_console  :=  _s "_Console";
   mW_lb_graph  :=  _s "_Graph";
-  mW_lb_im  :=  _s "_Im";
   mW_lb_settings  :=  _s "Se_ttings";
   mW_lb_initializing  :=  _s "Initializing";
   mW_lb_warning := _s "Warning!";
@@ -916,7 +910,6 @@ let load_messages () =
   mW_me_reconnect_to  :=  _s "Reconnect t_o";
   mW_me_settings  :=  !mW_lb_settings;
   mW_me_main_menu  :=  _s "Main me_nu";
-  mW_me_im  := _s "_Im";
   mW_me_restore := _s "R_estore";
 
 (* Settings Window *)
@@ -924,7 +917,6 @@ let load_messages () =
   cW_lb_main  :=  _s "_Main";
   cW_lb_interfaces  :=  _s "_Interfaces";
   cW_lb_tools  :=  _s "_Tools";
-  cW_lb_mlchat  :=  _s "M_Lchat";
   cW_lb_other  :=  _s "_Others";
   cW_lb_bittorrent  :=  _s "_BitTorrent";
   cW_lb_direct_connect  :=  _s "Di_rect Connect";
@@ -1111,7 +1103,6 @@ let icon_menu_rooms = "menu_rooms"
 let icon_menu_uploads = "menu_uploads"
 let icon_menu_console = "menu_console"
 let icon_menu_graph = "menu_graph"
-let icon_menu_im = "menu_im"
 let icon_menu_settings = "menu_settings"
 let icon_menu_quit = "menu_quit"
 let icon_menu_help = "menu_help"
@@ -1128,7 +1119,6 @@ let icon_menu_search_complex = "menu_search_complex"
 let icon_menu_search_freedb = "menu_search_freedb"
 let icon_menu_search_imdb = "menu_search_imdb"
 let icon_menu_interfaces = "menu_interfaces"
-let icon_menu_mlchat = "menu_mlchat"
 let icon_menu_tools = "menu_tools"
 let icon_menu_others = "menu_others"
 let icon_net_bittorrent = "net_bittorrent"
@@ -1268,7 +1258,7 @@ let help_string version = Printf.sprintf "
                       ========
 
 Release: %s
-Authors: MLDonkey project, http://www.mldonkey.net/
+Authors: MLDonkey project, http://www.mldonkey.org/
 
 This documentation file is now obsolete. Read the FAQ instead (either
 on the project WEB site or in the FAQ.html file).
@@ -1278,9 +1268,7 @@ For help, see:
 IRC channel: irc.freenode.net, chat #mldonkey
 
 Web sites:
-http://www.mldonkey.net/         Official site, bug reports
-http://www.mldonkeyworld.com/    English forum
-http://www.mldonkey.org/         German forum
+http://www.mldonkey.org          Official site
 
 Mailing-lists:
 mldonkey-users@nongnu.org
