@@ -60,6 +60,7 @@ type brand =
 | Brand_bittornado
 | Brand_bittorrentx
 | Brand_btplus
+| Brand_bitrocket
 | Brand_btslave
 | Brand_btugaxp
 | Brand_burst
@@ -103,6 +104,15 @@ type brand =
 | Brand_jvtorrent
 | Brand_retriever
 | Brand_osprey
+| Brand_rufus
+| Brand_tribler
+| Brand_cachelogic
+| Brand_electricsheep
+| Brand_qbittorrent
+| Brand_qt4
+| Brand_uleecher
+
+
 
 let brand_list = [ 
    ( Brand_unknown          , "unknown"                , "unk" ) ;
@@ -118,6 +128,7 @@ let brand_list = [
    ( Brand_bittornado       , "BitTornado"             , "trn" ) ;
    ( Brand_bittorrentx      , "BitTorrent X"           , "btx" ) ;
    ( Brand_btplus           , "BitTorrent Plus!"       , "plu" ) ;
+   ( Brand_bitrocket        , "BitRocket"              , "roc" ) ;
    ( Brand_btslave          , "BTSlave"                , "sla" ) ;
    ( Brand_btugaxp          , "BTugaXP"                , "uga" ) ;
    ( Brand_burst            , "Burst !"                , "brs" ) ;
@@ -161,6 +172,13 @@ let brand_list = [
    ( Brand_jvtorrent        , "JVtorrent"              , "jvt" ) ;
    ( Brand_retriever        , "Retriever"              , "ret" ) ;
    ( Brand_osprey           , "Osprey permaseed"       , "osp" ) ;
+   ( Brand_rufus            , "Rufus"                  , "ruf" ) ;
+   ( Brand_tribler          , "Tribler"                , "trb" ) ;
+   ( Brand_cachelogic       , "CacheLogic"             , "cl"  ) ;
+   ( Brand_electricsheep    , "Electric sheep"         , "els" ) ;
+   ( Brand_qbittorrent      , "qBittorrent"            , "qbt" ) ;
+   ( Brand_qt4              , "QT4"                    , "qt4" ) ;
+   ( Brand_uleecher         , "uLeecher!"              , "ul!" ) ;
   ] 
 
 let brand_count = List.length brand_list
@@ -174,6 +192,12 @@ let brand_to_string_short brand =
 let brand_to_int brand = 
   find_int_of_brand brand brand_list
 
+type tracker_status =
+  Enabled
+| Disabled of string
+| Disabled_mld of string
+| Disabled_failure of string
+
 type client = {
     client_client : client CommonClient.client_impl;
     mutable client_file : file;
@@ -185,7 +209,7 @@ type client = {
     mutable client_ranges_sent : (int64 * int64 * CommonSwarming.range) list;
     mutable client_range_waiting :
     (int64 * int64 * CommonSwarming.range) option;
-    mutable client_block : CommonSwarming.block option;
+    mutable client_chunk : (int * CommonSwarming.uploader_block list) option;
 
     mutable client_received_peer_id : bool;
     mutable client_sent_choke : bool; (* we sent a Choke to the client *)
@@ -205,7 +229,7 @@ type client = {
     mutable client_downloaded_rate :  Rate.t;
     mutable client_downloaded : int64;
     mutable client_uploaded : int64;
-    mutable client_optimist_time : int;
+    mutable client_connect_time : int;
 
     mutable client_blocks_sent : int list;
     mutable client_good : bool;
@@ -216,6 +240,13 @@ type client = {
     mutable client_incoming : bool;
     mutable client_registered_bitfield : bool;
     mutable client_last_optimist : int;
+
+    mutable client_dht : bool;
+    mutable client_cache_extension : bool;
+    mutable client_fast_extension : bool;
+    mutable client_utorrent_extension : bool;
+    mutable client_azureus_messaging_protocol : bool;
+
   }
 
 and tracker_info = {
@@ -231,7 +262,7 @@ and tracker_info = {
     mutable tracker_torrent_last_dl_req : int;
     mutable tracker_id : string;
     mutable tracker_key : string;
-    mutable tracker_enabled : bool;
+    mutable tracker_status : tracker_status;
   }
 
 and file = {
