@@ -80,7 +80,7 @@ let add_source file ip port serverIP serverPort =
                         (* without server, we can't request a callback *)
                         let s = Hashtbl.find servers_by_key serverIP in
                         if serverPort = s.server_port then
-                            Indirect_address ( serverIP, serverPort, id_of_ip ip, port, ip )
+                            Indirect_address ( serverIP, serverPort, id_of_ip ip, 0, Ip.null )
                         else
                             raise Not_found
                     with _ ->
@@ -112,7 +112,7 @@ let supports_eep cb =
   match cb with
     Brand_lmule | Brand_newemule | Brand_cdonkey |
     Brand_emuleplus | Brand_hydranode | Brand_mldonkey3 |
-    Brand_shareaza | Brand_amule | Brand_lphant -> true
+    Brand_shareaza | Brand_amule | Brand_lphant | Brand_verycd -> true
   | _ -> false
 
 let ban_client c sock msg = 
@@ -192,9 +192,13 @@ let _ =
           M.AvailableSlotReq Q.t);
         
         if !verbose then
-            lprintf_nl "New uploader %s"
-              (full_client_identifier c);
-        
+            lprintf_nl "New uploader %s%s%s"
+              (full_client_identifier c)
+	      (let slot_text = string_of_slot_kind (client_slot (as_client c)) true in
+		 if slot_text = "" then "" else Printf.sprintf "(%s)" slot_text)
+	      (match client_upload (as_client c) with
+		 None -> ""
+	       | Some f -> Printf.sprintf " for file %s" (CommonFile.file_best_name f))
     )  
   in
   client_ops.op_client_enter_upload_queue <- client_enter_upload_queue
@@ -262,7 +266,7 @@ let disconnect_client c reason =
           c.client_connect_time <- 0;
           (try Hashtbl.remove connected_clients c.client_md4 with _ -> ());
           (try CommonUploads.remove_pending_slot (as_client c) with _ -> ());
-          set_client_has_a_slot (as_client c) false;
+          set_client_has_a_slot (as_client c) NoSlot;
 (*          connection_failed c.client_connection_control; *)
           (try TcpBufferedSocket.close sock reason with _ -> ());
 
@@ -643,6 +647,7 @@ let parse_compatible_client num old_brand =
     | 6 -> Brand_hydranode
     | 10 -> Brand_mldonkey3
     | 20 -> Brand_lphant
+    | 240 -> Brand_verycd
     | _ -> Brand_unknown
 
 let parse_mod_version s c =
@@ -672,6 +677,16 @@ let update_client_from_tags c tags =
             DonkeyProtoClient.update_emule_proto_from_miscoptions1
             c.client_emule_proto i
           )
+      | Field_UNKNOWN "emule_miscoptions2" ->
+          for_int64_tag tag (fun i ->
+            DonkeyProtoClient.update_emule_proto_from_miscoptions2
+            c.client_emule_proto i
+          )
+      | Field_UNKNOWN "emule_compatoptions" ->
+          for_int_tag tag (fun i ->
+            DonkeyProtoClient.update_emule_proto_from_compatoptions
+            c.client_emule_proto i
+          )
       | Field_UNKNOWN "emule_version" ->
           for_int_tag tag (fun i ->
             c.client_emule_proto.emule_version <- i;
@@ -685,8 +700,9 @@ let update_client_from_tags c tags =
       | Field_UNKNOWN "mod_version" ->
           let s = to_lowercase (string_of_tag_value tag.tag_value) in 
           parse_mod_version s c
-      | _ -> ()
-
+      | _ -> 
+          if !verbose_msg_clienttags then
+            lprintf_nl "Unknown Emule tag: [%s] (update_client_from_tags)" (escaped_string_of_field tag)
   ) tags
     
 let update_emule_proto_from_tags c tags =
@@ -726,9 +742,15 @@ let update_emule_proto_from_tags c tags =
       | Field_UNKNOWN "mod_version" ->
           let s = to_lowercase (string_of_tag_value tag.tag_value) in 
           parse_mod_version s c;
+
+      | Field_UNKNOWN "os_info" ->
+          let s = to_lowercase (string_of_tag_value tag.tag_value) in 
+	  (match c.client_osinfo with
+	    Some _ -> ()
+	  | _ ->  if s <> "" then c.client_osinfo <- Some s)
       | _ -> 
-          if !verbose_msg_clients then
-            lprintf_nl "Unknown Emule tag: [%s]" (escaped_string_of_field tag)
+          if !verbose_msg_clienttags then
+            lprintf_nl "Unknown Emule tag: [%s] (update_emule_proto_from_tags)" (escaped_string_of_field tag)
   ) tags
 
 let fight_disguised_mods c =
@@ -742,6 +764,19 @@ let fight_disguised_mods c =
    if c.client_brand = Brand_emuleplus && c.client_brand_mod = Brand_mod_plus then
       c.client_brand_mod <- Brand_mod_unknown
 
+let request_osinfo c =
+  if c.client_emule_proto.emule_osinfosupport = 1 && not c.client_osinfo_sent then
+    begin
+      let emule_osinfo = {
+	emule_info with
+	DonkeyProtoClient.EmuleClientInfo.protversion = 255;
+	DonkeyProtoClient.EmuleClientInfo.tags = [
+	  string_tag (Field_UNKNOWN "os_info") (String2.upp_initial Autoconf.system);
+	]} in
+      client_send c (DonkeyProtoClient.EmuleClientInfoReq emule_osinfo);
+      c.client_osinfo_sent <- true
+    end
+
 let rec query_id ip port id =
   let client_ip = client_ip None in
 
@@ -790,15 +825,13 @@ let shared_of_file file =
     | Some sh -> Some (as_shared sh)
 
 let query_view_files c =
-  if client_browsed_tag land client_type c <> 0 then begin
-      if last_time () > c.client_next_view_files then begin
-          c.client_next_view_files <- last_time () + 3600 * 6;
-          client_send c (
-            let module M = DonkeyProtoClient in
-            let module C = M.ViewFiles in
-            M.ViewFilesReq C.t);
-        end
-    end
+  if CommonClient.is_must_browse (as_client c) then begin
+    CommonClient.set_not_must_browse (as_client c);
+    client_send c (
+      let module M = DonkeyProtoClient in
+      let module C = M.ViewFiles in
+      M.ViewFilesReq C.t);
+  end
 
 (* client is valid if it's not us or if it's not yet connected *)
 let is_valid_client md4 =
@@ -914,7 +947,7 @@ let send_pending_messages c sock =
   c.client_pending_messages <- []
   
 let init_client_after_first_message sock c = 
-  (* we read something on socket so ip is now know for socket *)
+  (* we read something on socket so ip is now known for socket *)
   c.client_ip <- peer_ip sock;
   (* Add the Connected tag and when needed the NoLimit tag *)
   let t = client_type c lor client_initialized_tag in
@@ -1050,14 +1083,14 @@ let get_server_ip_port () =
 let process_mule_info c t =
   update_emule_proto_from_tags c t;
   update_emule_release c;
-  if !!enable_sui
+  client_must_update c;
+  if sec_ident_enabled ()
       && (c.client_md4 <> Md4.null) 
       && (c.client_sent_challenge == Int64.zero) 
       && (c.client_emule_proto.emule_secident > 0) 
   then begin
-    if !verbose_msg_clients then begin
+    if !verbose_msg_clients then
       lprintf_nl "%s [process_mule_info] [verify_ident]" (full_client_identifier c);
-    end;
     verify_ident c
   end
 
@@ -1109,7 +1142,7 @@ let client_to_client for_files c t sock =
       
       init_client_after_first_message sock c;
       
-      set_client_has_a_slot (as_client c) false;
+      set_client_has_a_slot (as_client c) NoSlot;
       
       let module CR = M.Connect in
 
@@ -1179,14 +1212,10 @@ let client_to_client for_files c t sock =
       if !!emule_mods_count then
         identify_client_brand_mod c t.CI.tags;
       
-      (* TODO : remove this comment 
-          ERR! i think the peer support eep if it send an emule client info
-          PLUS this message is sent _before_ we received an md4 so 
-          client_brand is unknown here.
-      if supports_eep c.client_brand then  begin
-      *)
-          let module E = M.EmuleClientInfo in
-          client_send c (M.EmuleClientInfoReplyReq emule_info)
+      let module E = M.EmuleClientInfo in
+      client_send c (M.EmuleClientInfoReplyReq emule_info);
+      request_osinfo c;
+
   
   | M.EmuleClientInfoReplyReq t -> 
       
@@ -1272,7 +1301,7 @@ let client_to_client for_files c t sock =
             lprintf_nl "Exception in ViewFilesReply %s"
               (Printexc2.to_string e); 
       end;
-  
+
   | M.AvailableSlotReq _ ->
       set_lifetime sock active_lifetime;
       set_rtimeout sock !!queued_timeout; 
@@ -1365,8 +1394,11 @@ other one for unlimited sockets.  *)
         end else *)
       CommonUploads.add_pending_slot (as_client c);
       if !verbose_upload then
-          lprintf_nl "donkeyClient: uploader couldn't get a slot: %s"
-            (full_client_identifier c);
+          lprintf_nl "added to pending slots: %s %s"
+            (full_client_identifier c) 
+	    (match client_upload (as_client c) with
+	       None -> ""
+	     | Some f -> CommonFile.file_best_name f);
 (*      end *)
   
   | M.CloseSlotReq _ ->
@@ -1406,23 +1438,8 @@ other one for unlimited sockets.  *)
           c.client_rating <- c.client_rating + 1;
           
           client_has_file c file;
-          
-          begin
-            let ips = 
-              try
-                List.assoc t.Q.name file.file_filenames
-              with Not_found ->
-                  let ips = noips() in
-                  file.file_filenames <- file.file_filenames @ [t.Q.name, ips];
-                  update_best_name file;
-                  ips
-            in
-            if not (List.mem c.client_ip ips.ips) then begin
-                ips.ips <- c.client_ip :: ips.ips;
-                ips.nips <- 1 + ips.nips;
-              end
-          end;
-          
+	  add_file_filenames (as_file file) t.Q.name;
+
           if file_size file <= block_size then begin
               client_is_useful c file (Bitv.create 1 true)
             end else begin
@@ -1437,6 +1454,28 @@ other one for unlimited sockets.  *)
         with _ -> ()
       end  
   
+  | M.EmuleFileDescReq t ->
+      begin
+        match c.client_last_file_req_md4 with
+          Some md4 -> 
+            begin
+              try 
+                let file = find_file md4 in
+                let module Q = M.EmuleFileDesc in
+                let slen = String.length t.Q.comment in
+                if slen > 0 && slen <= !!max_comment_length && (!is_not_comment_spam) t.Q.comment then begin
+                  (* Disallow dups from single IP, but allow comment updates *)
+                  file.file_comments <- List.filter (fun (i,_,_,_) -> i <> c.client_ip) file.file_comments;
+                  if List.length file.file_comments < !!max_comments_per_file then begin
+                    file.file_comments <- (c.client_ip, c.client_name, t.Q.rating, (intern t.Q.comment)) :: file.file_comments;
+                    file_must_update file;
+                  end;
+                end
+              with _ -> ()
+            end
+         | None -> ()
+      end
+
   | M.QueryChunksReplyReq t ->
       let module Q = M.QueryChunksReply in
       begin
@@ -1688,7 +1727,7 @@ is checked for the file.
                 shared_must_update_downloaded (as_shared impl);
                 impl.impl_shared_requests <- impl.impl_shared_requests + 1);
           request_for c file sock;
-          set_client_upload (as_client c) (shared_of_file file);
+          set_client_upload (as_client c) (as_file file);
           client_send c (
             let module Q = M.QueryFileReply in
             let filename = file_best_name file in
@@ -1721,7 +1760,7 @@ is checked for the file.
       end
 
   | M.EmuleSignatureReq t ->
-      if !!enable_sui then 
+      if sec_ident_enabled () then 
       begin
       let module Q = M.EmuleSignatureReq in
 
@@ -1776,7 +1815,7 @@ is checked for the file.
       end
 
   | M.EmulePublicKeyReq t ->
-      if !!enable_sui then 
+      if sec_ident_enabled () then 
       begin
       let module Q = M.EmulePublicKeyReq in
         (match c.client_public_key with 
@@ -1801,12 +1840,11 @@ is checked for the file.
         );
       end
        else
-      if !verbose_msg_clients then begin
-        lprintf_nl "%s [EPubKeyReq] [DISABLED]" (full_client_identifier c) ;
-      end
+      if !verbose_msg_clients then
+        lprintf_nl "%s [EPubKeyReq] [DISABLED]" (full_client_identifier c);
 
   | M.EmuleSecIdentStateReq t ->
-      if !!enable_sui then 
+      if sec_ident_enabled () then 
       begin
       let module Q = M.EmuleSecIdentStateReq in
 
@@ -1829,9 +1867,8 @@ is checked for the file.
           then send_signature c;
 
       end else
-      if !verbose_msg_clients then begin
-        lprintf_nl "%s [ESecIdentStateReq] [DISABLED]" (full_client_identifier c) ;
-      end
+      if !verbose_msg_clients then
+        lprintf_nl "%s [ESecIdentStateReq] [DISABLED]" (full_client_identifier c);
 
   | M.EmuleRequestSourcesReplyReq t ->
 (*      lprintf "Emule sent sources\n";  *)
@@ -1977,7 +2014,7 @@ end else *)
       let module Q = M.QueryBloc in
       let file = find_file  t.Q.md4 in
       let prio = (file_priority file) in
-      let client_upload_lifetime = ref ((maxi 0 !!upload_lifetime) * 60) in
+      let client_upload_lifetime = ref ((max 0 !!upload_lifetime) * 60) in
       begin
         
         if !!dynamic_upload_lifetime
@@ -2024,7 +2061,7 @@ end else *)
         new_chunk up t.Q.start_pos2 t.Q.end_pos2;
         new_chunk up t.Q.start_pos3 t.Q.end_pos3;
         c.client_upload <- Some up;
-        set_client_upload (as_client c) (shared_of_file file);
+        set_client_upload (as_client c) (as_file file);
         if not waiting && !CommonUploads.has_upload = 0 then begin
             CommonUploads.ready_for_upload (as_client c);
             up.up_waiting <- true
@@ -2120,18 +2157,19 @@ let init_client sock c =
 (*  c.client_block <- None; *)
 (*  c.client_zones <- []; *)
   c.client_file_queue <- [];
-  set_client_has_a_slot (as_client c) false;
+  set_client_has_a_slot (as_client c) NoSlot;
   c.client_upload <- None;
   c.client_rank <- 0;
   c.client_requests_received <- 0;
   c.client_requests_sent <- 0;
   c.client_slot <- SlotNotAsked
         
-let read_first_message overnet m sock =
+let read_first_message overnet server m sock =
   let module M = DonkeyProtoClient in
     
-  if !verbose_msg_clients then begin
-      lprintf_nl "Message from incoming client";
+  if (not server && !verbose_msg_clients) || (server && !verbose_msg_servers) then begin
+      lprintf_nl "Message from incoming %s %s:%d"
+	(if server then "server" else "client") (Ip.to_string (peer_ip sock)) (peer_port sock);
       M.print m;
     end;
 
@@ -2229,7 +2267,8 @@ let read_first_message overnet m sock =
               end
       end;
 
-      if supports_eep c.client_brand then
+(* Lugdunum servers are not interested in our EmuleClientInfo *)
+      if supports_eep c.client_brand && not server then
         begin
           (* lprintf "Emule Extended Protocol query\n"; *)
           let module M = DonkeyProtoClient in
@@ -2237,6 +2276,7 @@ let read_first_message overnet m sock =
           client_send c (M.EmuleClientInfoReq emule_info)
         end;
       
+      request_osinfo c;
       client_send c (
         let module M = DonkeyProtoClient in
         let module C = M.Connect in
@@ -2256,7 +2296,8 @@ let read_first_message overnet m sock =
               C.md4 = !!client_md4;
               C.ip = client_ip (Some sock);
               C.port = !!donkey_port;
-              C.tags = !client_to_client_tags;
+              (* Lugdunum servers need fewer infos than clients *)
+              C.tags = if server then !client_to_server_reply_tags else !client_to_client_tags;
               C.server_info = Some (get_server_ip_port ());
               C.left_bytes = left_bytes; 
               C.hash_len = 16;
@@ -2274,7 +2315,14 @@ let read_first_message overnet m sock =
   | M.NewUserIDReq _ ->
       lprintf_nl "NewUserIDReq: "; M.print m; 
       None
-  
+
+  | M.EmulePortTestReq t ->
+      porttest_sock := Some sock;
+      set_closer sock (fun _ _ -> porttest_sock := None);
+      set_lifetime sock 30.;
+      write_string sock (client_msg_to_string (emule_proto ()) m);
+      None
+
   | _ -> 
       if !verbose_unknown_messages then
         begin
@@ -2459,11 +2507,13 @@ let client_connection_handler overnet t event =
           (if is_connecting_server then
             ( try 
                 let s = Hashtbl.find servers_by_key from_ip in
-                Printf.sprintf " %s (%s)" s.server_name (Ip.to_string s.server_ip)
+                set_server_state s Connected_initiating;
+                Printf.sprintf " %s (%s)" s.server_name (string_of_server s)
               with _ ->
                 try 
                   let s = Hashtbl.find servers_by_key connecting_server in
-                  Printf.sprintf " %s (%s)" s.server_name (Ip.to_string s.server_ip)
+                  set_server_state s Connected_initiating;
+                  Printf.sprintf " %s (%s)" s.server_name (string_of_server s)
                 with _ -> "Unknown server"
             )
            else ""
@@ -2494,7 +2544,7 @@ let client_connection_handler overnet t event =
               );
               (try
                   set_reader sock 
-                    (DonkeyProtoCom.client_handler2 c (read_first_message overnet)
+                    (DonkeyProtoCom.client_handler2 c (read_first_message overnet is_connecting_server)
                     (client_to_client []));
                 
                 with e -> lprintf_nl "Exception %s in init_connection"
@@ -2550,6 +2600,7 @@ let _ =
 (* TODO build the extension if needed *)
             M.QueryFile.emule_extension = emule_extension;
           });
+          c.client_last_file_req_md4 <- Some file.file_md4;
             let know_file_chunks = List.exists (fun (f,_,_) -> f == file) c.client_file_queue in
             if not know_file_chunks then
               DonkeyProtoCom.client_send c (
@@ -2572,7 +2623,7 @@ let _ =
         | Invalid_address _ -> ()
         | Indirect_address (server_ip, server_port, id, port, real_ip) ->
 
-       if low_id server_ip && Ip.reachable server_ip then
+       if Ip.reachable server_ip then
               query_id server_ip server_port id; 
                   
       with e -> 
@@ -2610,7 +2661,7 @@ a FIFO from where they are removed after 30 minutes. What about using
         (CommonClient.as_client c.client_client);
       
       with
-  Not_found -> ()
+      | Not_found -> ()
       | e -> 
         if !verbose then
           lprintf_nl "add_location: exception %s" (Printexc2.to_string e)
@@ -2625,9 +2676,9 @@ a FIFO from where they are removed after 30 minutes. What about using
         (CommonClient.as_client c.client_client);
         
       with
-  Not_found -> ()
+      | Not_found -> ()
       | e -> 
         if !verbose then
           lprintf_nl "remove_location for file_md4 %s: exception %s"
-      file_uid (Printexc2.to_string e)
+	    file_uid (Printexc2.to_string e)
   )
