@@ -46,16 +46,6 @@ open DonkeyClient
 open CommonGlobals
 open DonkeyStats
 
-(* prints a new logline with date, module and starts newline *)
-let lprintf_nl () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf_nl2
-
-(* prints a new logline with date, module and does not start newline *)
-let lprintf_n () =
-  lprintf "%s[EDK] "
-    (log_time ()); lprintf
-
 let msg_block_size_int = 10240
 let msg_block_size = Int64.of_int msg_block_size_int
 let upload_buffer = String.create msg_block_size_int
@@ -89,8 +79,8 @@ module NewUpload = struct
 (*      let len_int = Int32.to_int len in *)
       try
 	if !verbose_upload then
-	  lprintf_nl () "send_small_block (%s) %Ld %d"
-	    (full_client_identifier c)
+	  lprintf_nl "Sending %s to %s, begin %Ld len %d"
+	    (file_best_name file) (full_client_identifier c)
             (begin_pos) (len_int);
         
         let msg =  
@@ -124,9 +114,9 @@ module NewUpload = struct
         write_string sock upload_buffer;
         check_end_upload c sock
       with
-	End_of_file -> lprintf_nl () "Can not send file %s to %s, file removed?"
+  End_of_file -> lprintf_nl "Can not send file %s to %s, file removed?"
 			 (file_best_name file) (full_client_identifier c)
-      | e -> if !verbose then lprintf_nl ()
+      | e -> if !verbose then lprintf_nl
 	       "Exception %s in send_small_block" (Printexc2.to_string e)
     
     let rec send_client_block c sock per_client =
@@ -140,19 +130,19 @@ module NewUpload = struct
               end else
             let max_len = up.up_end_chunk -- up.up_pos in
             let max_len = Int64.to_int max_len in
-            let msg_block_size_int = mini msg_block_size_int per_client in
+            let msg_block_size_int = min msg_block_size_int per_client in
             if max_len <= msg_block_size_int then
 (* last block from chunk *)
               begin
                 if !verbose_upload then
-                    lprintf_nl () "END OF CHUNK (%d) %Ld" max_len up.up_end_chunk; 
+                    lprintf_nl "End of chunk (%d) %Ld %s" max_len up.up_end_chunk (file_best_name up.up_file);
                 send_small_block c sock up.up_file up.up_pos max_len;
                 up.up_chunks <- chunks;
                 let per_client = per_client - max_len in
                 match chunks with
                   [] -> 
                     if !verbose_upload then
-                        lprintf_nl () "NO CHUNKS";
+                        lprintf_nl "NO CHUNKS";
                     c.client_upload <- None;
                 | (begin_pos, end_pos) :: _ ->
                     up.up_pos <- begin_pos;
@@ -175,7 +165,7 @@ module NewUpload = struct
       do_if_connected  c.client_source.DonkeySources.source_sock (fun sock ->
 (*    lprintf "upload_to_client %d connected\n"  (maxi max_msg_size size); *)
           
-          let size = mini max_msg_size size in
+          let size = min max_msg_size size in
           send_client_block c sock size;
            (match c.client_upload with
               None -> ()
