@@ -42,6 +42,11 @@ open DonkeyTypes
 open DonkeyOptions
 open CommonOptions
 
+let log_prefix = "[EDK]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
 (*************************************************************
 
 Define the instances of the plugin classes, that we be filled
@@ -51,7 +56,8 @@ plugin.
 **************************************************************)
 
 let network = CommonNetwork.new_network "ED2K" "Donkey"
-         [
+  ~comment:(if Autoconf.donkey_sui_works () then "SUI" else "noSUI")
+  [
     NetworkHasServers;
     NetworkHasSearch;
     NetworkHasUpload;
@@ -120,13 +126,12 @@ let tag_client = 200
 let tag_server = 201
 let tag_file   = 202
 
-let page_size = 4096L
-
 let donkey_download_counter = ref Int64.zero
 let donkey_upload_counter = ref Int64.zero
 
 let client_to_client_tags = ref ([] : tag list)
 let client_to_server_tags = ref ([] : tag list)
+let client_to_server_reply_tags = ref ([] : tag list)
 let emule_info =
   let module E = DonkeyProtoClient.EmuleClientInfo in
   {
@@ -135,6 +140,8 @@ let emule_info =
     E.tags = [];
   }
 
+let sec_ident_enabled () = !!enable_sui && (Autoconf.donkey_sui_works ())
+
 let overnet_connectreply_tags = ref ([] :  tag list)
 let overnet_connect_tags = ref ([] :  tag list)
 
@@ -179,8 +186,7 @@ let max_file_groups = 1000
 let master_server = ref (None: DonkeyTypes.server option)
 let udp_sock = ref (None: UdpSocket.t option)
 let listen_sock = ref (None : TcpServerSocket.t option)
-let reversed_sock = ref (None : TcpServerSocket.t option)
-let new_shared = ref false
+let porttest_sock = ref (None : TcpBufferedSocket.t option)
 
 (*************************************************************************)
 (*                                                                       *)
@@ -199,6 +205,8 @@ let clients_by_kind = H.create 127
 let clients_root = ref []
 let servers_by_key = Hashtbl.create 127
 let servers_list = ref ([] : server list)
+let walker_list = ref ([] : server list)
+let delayed_list = ref ([] : server list)
 
 (* let remaining_time_for_clients = ref (60 * 15) *)
 
@@ -225,7 +233,6 @@ in the shared_files_info *)
 let shared_files_info = (Hashtbl.create 127
     : (string * int64 * float, shared_file_info) Hashtbl.t)
 let shared_files = ref ([] : file_to_share list)
-let new_shared_files = ref []
 
 let udp_servers_replies = (Hashtbl.create 127 : (Md4.t, server) Hashtbl.t)
 
@@ -303,10 +310,10 @@ let update_best_name file =
       match impl.impl_file_probable_name with
         None -> ()
       | Some best_name ->
-          set_file_best_name file best_name
+          set_file_best_name file best_name "" 0
     with Not_found -> ()
 
-let new_file file_diskname file_state md4 file_size filenames writable =
+let new_file file_diskname file_state md4 file_size filename writable user =
 
   try
       let file = find_file md4 in
@@ -343,6 +350,7 @@ let new_file file_diskname file_state md4 file_size filenames writable =
         lprintf_nl "New file with md4: %s" (Md4.to_string md4);
 
       let t =
+(*  emulate_sparsefiles does not work, temporarily disabled
         if
 (* Don't use this for shared files ! *)
           writable &&
@@ -353,6 +361,7 @@ let new_file file_diskname file_state md4 file_size filenames writable =
         then
           Unix32.create_sparsefile file_diskname writable
         else
+*)
           Unix32.create_diskfile file_diskname writable
       in
       let file_size =
@@ -372,8 +381,8 @@ let new_file file_diskname file_state md4 file_size filenames writable =
       (try
         Unix32.remove t
        with e ->
-    lprintf_nl "Unix32.remove %s exception %s"
-      (file_diskname) (Printexc2.to_string e));
+	 lprintf_nl "Unix32.remove %s exception %s"
+	   (file_diskname) (Printexc2.to_string e));
       Unix32.destroy t;
       failwith (Printf.sprintf "file size %s is too big, exception: %s"
         (size_of_int64 file_size) (Printexc2.to_string e))
@@ -388,20 +397,23 @@ let new_file file_diskname file_state md4 file_size filenames writable =
           file_swarmer = None;
           file_nchunks = get_nchunks file_size;
           file_nchunk_hashes = get_nchunk_hashes file_size;
-          file_filenames = filenames;
           file_computed_md4s = Array.of_list md4s;
           file_format = FormatNotComputed 0;
           file_sources = DonkeySources.create_file_sources_manager
-            (Md4.to_string md4)
+            (Md4.to_string md4);
+          file_comments = [];
         }
       and file_impl = {
           dummy_file_impl with
+          impl_file_owner = user;
+          impl_file_group = user.user_default_group;
           impl_file_val = file;
           impl_file_ops = file_ops;
           impl_file_age = last_time ();
           impl_file_size = file_size;
           impl_file_fd = Some t;
           impl_file_best_name = Filename.basename file_diskname;
+          impl_file_filenames = (if filename = "" then [] else [filename]);
           impl_file_last_seen = last_time () - 100 * 24 * 3600;
         }
       in
@@ -415,27 +427,12 @@ let new_file file_diskname file_state md4 file_size filenames writable =
             let swarmer = CommonSwarming.create kernel (as_file file) block_size
             in
             file.file_swarmer <- Some swarmer;
-(*
-        CommonSwarming.set_writer swarmer (fun offset s pos len ->
-(*
-      lprintf "DOWNLOADED: %d/%d/%d\n" pos len (String.length s);
-      AnyEndian.dump_sub s pos len;
-*)
-
-            if !!CommonOptions.buffer_writes then
-              Unix32.buffered_write_copy t offset s pos len
-            else
-              Unix32.write  t offset s pos len
-        ); *)
             CommonSwarming.set_verifier swarmer
               (if md4s = [] then VerificationNotAvailable else
                 Verification (Array.of_list (List.map (fun md4 -> Ed2k md4) md4s))
             );
             CommonSwarming.set_verified swarmer (fun nblocks num ->
-                if nblocks = 1 then begin
-                    new_shared_files := file :: !new_shared_files;
-                    file_must_update file
-                  end)
+                if nblocks = 1 then file_must_update file)
       );
 
       update_best_name file;
@@ -471,7 +468,10 @@ let low_id ip =
 let is_black_address ip port =
   !!black_list && not (low_id ip) && (
 (* lprintf "is black ="; *)
-    not (Ip.reachable ip) || (Ip.matches ip !!server_black_list) ||
+    not (Ip.reachable ip) || 
+    (match Ip_set.match_ip !server_black_list_set ip with
+     | Some br -> true 
+     | None -> false) ||
     (List.mem port !!port_black_list) ||
     (match !Ip.banned ip with
         None -> false
@@ -480,7 +480,7 @@ let is_black_address ip port =
             lprintf_nl "%s:%d blocked: %s" (Ip.to_string ip) port reason;
           true))
 
-let new_server ip port score =
+let new_server ip port =
   let key = (ip) in
   try
     let found = Hashtbl.find servers_by_key key in
@@ -497,34 +497,50 @@ let new_server ip port score =
         server_port = port;
         server_realport = None;
         server_sock = NoConnection;
-        server_nqueries = 0;
         server_search_queries = Fifo.create ();
         server_users_queries = Fifo.create ();
         server_connection_control = new_connection_control ();
-        server_score = score;
+        server_score = 5;
         server_tags = [];
-        server_nfiles = Int64.zero;
-        server_nusers = Int64.zero;
+        server_nfiles = None;
+        server_nusers = None;
         server_name = "";
         server_description = "";
         server_banner = "";
         server_users = [];
         server_master = false;
         server_preferred = false;
-        server_last_message = 0;
         server_queries_credit = 0;
         server_waiting_queries = [];
         server_sent_all_queries = false;
         server_id_requests = Fifo.create ();
         server_flags = 0;
         server_has_zlib = false;
+        server_has_newtags = false;
+        server_has_unicode = false;
+        server_has_related_search = false;
+        server_has_tag_integer = false;
+        server_has_largefiles = false;
         server_version = "";
-        server_lowid_users = 0L;
-        server_soft_limit = 0L;
-        server_hard_limit = 0L;
-        server_max_users = 0L;
+        server_lowid_users = None;
+        server_soft_limit = None;
+        server_hard_limit = None;
+        server_obfuscation_tcp = None;
+        server_obfuscation_udp = None;
+        server_sent_shared = [];
+        server_max_users = None;
         server_last_ping = 0.;
+        server_next_ping = 0.;
+        server_descping_counter = 0;
         server_ping = 0;
+        server_failed_count = 0;
+        server_udp_ping_challenge = None;
+        server_udp_desc_challenge = None;
+        server_has_get_sources = false;
+        server_has_get_files = false;
+        server_has_get_sources2 = false;
+        server_dynip = "";
+        server_auxportslist = "";
 
       }
       and server_impl =
@@ -550,6 +566,8 @@ let remove_server ip port =
   try
     Hashtbl.remove servers_by_key key;
     servers_list := List2.removeq s !servers_list ;
+    walker_list := List2.removeq s !walker_list;
+    delayed_list := List2.removeq s !delayed_list;
     (match s.server_sock with
         NoConnection -> ()
       | ConnectionWaiting token -> cancel_token token
@@ -564,24 +582,17 @@ let dummy_client =
       client_upload = None;
       client_kind = Direct_address (Ip.null, 0);
       client_source = DonkeySources.dummy_source;
-(*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-(*      client_last_filereqs = 0; *)
-(*      client_chunks = [||]; *)
       client_download = None;
-(*      client_block = None; *)
-(*      client_zones = []; *)
-(*      client_connection_control =  new_connection_control_recent_ok ( ()); *)
       client_file_queue = [];
       client_tags = [];
       client_name = "";
       client_all_files = None;
-      client_next_view_files = last_time () - 1;
-(*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
+      client_osinfo = None;
       client_checked = false;
       client_connected = false;
       client_downloaded = Int64.zero;
@@ -604,6 +615,8 @@ let dummy_client =
       client_sent_challenge = Int64.zero;
       client_public_key = None;
       client_sui_verified = None;
+      client_last_file_req_md4 = None;
+      client_osinfo_sent = false;
       } and
     client_impl = {
       dummy_client_impl with
@@ -616,32 +629,25 @@ let dummy_client =
 
 let create_client key =
   let module D = DonkeyProtoClient in
-  let s = DonkeySources.find_source_by_uid key in
+  let s = DonkeySources.find_source_by_uid (match key with
+      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null) 
+      | _ -> key) in
   let rec c = {
-(*      dummy_client with *)
-
       client_client = client_impl;
-(*      client_connection_control =  new_connection_control_recent_ok ( ()); *)
-      client_next_view_files = last_time () - 1;
       client_kind = key;
       client_upload = None;
       client_source = s;
-(*      client_sock = NoConnection; *)
       client_ip = Ip.null;
       client_md4 = Md4.null;
-(*      client_last_filereqs = 0; *)
-(*      client_chunks = [||]; *)
       client_download = None;
-(*      client_block = None; *)
-(*      client_zones = []; *)
       client_file_queue = [];
       client_tags = [];
       client_name = "";
       client_all_files = None;
-(*      client_all_chunks = ""; *)
       client_rating = 0;
       client_brand = Brand_unknown;
       client_brand_mod = Brand_mod_unknown;
+      client_osinfo = None;
       client_checked = false;
       client_connected = false;
       client_downloaded = Int64.zero;
@@ -664,8 +670,9 @@ let create_client key =
       client_sent_challenge = Int64.zero;
       client_public_key = None;
       client_sui_verified = None;
-      } and
-    client_impl = {
+      client_last_file_req_md4 = None;
+      client_osinfo_sent = false;
+      } and    client_impl = {
       dummy_client_impl with
       impl_client_val = c;
       impl_client_ops = client_ops;
@@ -678,17 +685,27 @@ let create_client key =
   clients_root := c :: !clients_root;
   c
 
+exception ClientFound of client
+let find_client_by_key key =
+  try
+    H.iter (fun c ->
+        if (match c.client_kind with
+      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null)
+      | _ -> c.client_kind) = (match key with
+      Indirect_address (server_ip, server_port, id, port, real_ip) -> Indirect_address (server_ip, server_port, id, 0, Ip.null) 
+      | _ -> key) then raise (ClientFound c)
+    ) clients_by_kind;
+    raise Not_found
+  with ClientFound c -> c
+
 let new_client key =
   try
-    H.find clients_by_kind { dummy_client with client_kind = key }
+      find_client_by_key key
   with _ ->
       create_client key
 
 let create_client = ()
 
-let find_client_by_key key =
-  H.find clients_by_kind { dummy_client with client_kind = key }
-
 let client_type c =
   client_type (as_client c)
 
@@ -709,6 +726,8 @@ let string_of_client c =
     | Invalid_address _ -> ""
   )
 
+let string_of_server s =
+  Printf.sprintf "%s:%d" (Ip.to_string s.server_ip) s.server_port
 
 let set_client_name c name md4 =
   if name <> c.client_name || c.client_md4 <> md4 then begin
@@ -820,7 +839,6 @@ end;
   Printf.bprintf buf "  udp_servers_list: %d\n" (List.length !udp_servers_list);
   Printf.bprintf buf "  interesting_clients: %d\n" (List.length !interesting_clients);
   Printf.bprintf buf "  shared_files: %d\n" (List.length !shared_files);
-  Printf.bprintf buf "  new_shared_files: %d\n" (List.length !new_shared_files);
   Printf.bprintf buf "  servers_by_key: %d\n" (Hashtbl.length servers_by_key);
   Printf.bprintf buf "  banned_ips: %d\n" (Hashtbl.length banned_ips);
   Printf.bprintf buf "  old_requests: %d\n" (Hashtbl.length old_requests);
@@ -884,11 +902,6 @@ let last_connected_server () =
         [] -> raise Not_found
       | s :: _ -> s
 
-let all_servers () =
-  Hashtbl.fold (fun key s l ->
-      s :: l
-  ) servers_by_key []
-
 let string_of_file_state s =
   match  s with
   | FileDownloading -> "File Downloading"
@@ -905,6 +918,8 @@ let left_bytes = "MLDK"
 let overnet_server_ip = ref Ip.null
 let overnet_server_port = ref 0
 
+let overnet_port_info = ref 0
+let kademlia_port_info = ref 0
 
 
 (*************************************************************
@@ -968,49 +983,38 @@ let clean_join_queue_tables () =
   ) list
 
 let client_public_key = ref ""
-
+let key_check_started = ref false
 
 let _ =
   option_hook client_private_key (fun _ ->
-    if Autoconf.donkey_sui_works () then begin
-    client_public_key := DonkeySui.SUI.load_key (!!client_private_key);
-    (
-    let key_checked = ref false in
-    let key_check_again = ref false in
-    let rec check_client_private_key () =
-      key_checked := true;
-      if not (try String.sub !!client_private_key 0 4 = "MIIB" with e -> false) then
-        if !key_check_again then
-    begin
-      lprintf_nl "can not create proper client_private_key, exiting...";
-      exit 70
-    end
-  else
-          begin
-            lprintf_nl "bad client_private_key detected, creating new key";
-      set_simple_option donkey_ini "client_private_key" (DonkeySui.SUI.create_key ());
-      key_check_again := true
-    end
-    in
-    if not !key_checked then check_client_private_key ();
-    if !key_check_again then
+    if Autoconf.donkey_sui_works () then
       begin
-  lprintf_nl "re-checking private key";
-  check_client_private_key ()
+	if not (try String.sub !!client_private_key 0 4 = "MIIB" with e -> false) then
+	  if !key_check_started then
+	    begin
+	      let s1 =
+		Printf.sprintf "can not create valid client_private_key, bad value found: %s"
+		  !!client_private_key in
+	      let s2 = "CryptoPP code seems not to work properly, do not use insane CFLAGS, exiting..." in
+	      Printf.eprintf "%s[EDK] %s\n%!" (log_time ()) s1;
+	      Printf.eprintf "%s[EDK] %s\n%!" (log_time ()) s2;
+	      lprintf_nl "%s" s1;
+	      lprintf_nl "%s" s2;
+	      exit 70
+	    end
+	  else
+	    begin
+	      key_check_started := true;
+	      lprintf_nl "found bad client_private_key: %s, creating new key..." !!client_private_key;
+	      client_private_key =:= (DonkeySui.SUI.create_key ());
+	    end
+	else
+	  begin
+	    client_public_key := DonkeySui.SUI.load_key (!!client_private_key);
+	  end
       end
-    );
-    end
   )
 
-let server_accept_multiple_getsources s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_getsources) <> 0
-
-let server_send_multiple_replies s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.multiple_replies) <> 0
-
-let server_send_getsources2 s =
-  (s.server_flags land DonkeyProtoUdp.PingServerReplyUdp.getsources2) <> 0
-
 let full_client_identifier c =
     Printf.sprintf "%s (%s%s) '%s'"
       (Ip.to_string c.client_ip)
