@@ -21,6 +21,8 @@ open Int64ops
 open Printf2
 open Md4
 open Options
+open LittleEndian
+open AnyEndian
 
 open BasicSocket
 open TcpBufferedSocket
@@ -55,8 +57,20 @@ open CommonOptions
 open DonkeyStats
 open DonkeyUdp
 
+open Gettext
+
+let _s x = _s "DonkeyInteractive" x
+let _b x = _b "DonkeyInteractive" x
+
+let porttest_result = ref PorttestNotStarted
+
 module VB = VerificationBitmap
 
+let log_prefix = "[EDK]"
+
+let lprintf_nl fmt =
+  lprintf_nl2 log_prefix fmt
+
 let result_name r =
   match r.result_names with
     [] -> None
@@ -64,7 +78,7 @@ let result_name r =
 
 
 let op_file_proposed_filenames file =
-  List2.tail_map fst file.file_filenames
+  file.file_file.impl_file_filenames
 
 let reconnect_all file =
   DonkeyProtoOvernet.Overnet.recover_file file;
@@ -99,12 +113,47 @@ let load_server_met filename =
           let server = check_add_server r.S.ip r.S.port in
           List.iter (fun tag ->
               match tag with
-                { tag_name = Field_UNKNOWN "name"; tag_value = String s } ->
+              |  { tag_name = Field_UNKNOWN "name"; tag_value = String s } ->
                   server.server_name <- s;
               |  { tag_name = Field_UNKNOWN "description" ; tag_value = String s } ->
                   server.server_description <- s
-              | _ -> ()
-          ) r.S.tags
+              |  { tag_name = Field_UNKNOWN "version" ; tag_value = Uint64 s } ->
+                  server.server_version <- Printf.sprintf "%d.%d"
+					    ((Int64.to_int s) lsr 16) ((Int64.to_int s) land 0xFFFF)
+              |  { tag_name = Field_UNKNOWN "ping" ; tag_value = Uint64 s } ->
+                  server.server_ping <- (Int64.to_int s)
+              |  { tag_name = Field_UNKNOWN "dynip" ; tag_value = String s } ->
+                  server.server_dynip <- s
+              |  { tag_name = Field_UNKNOWN "users" ; tag_value = Uint64 s } ->
+                  (match server.server_nusers with
+		  | None -> server.server_nusers <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "files" ; tag_value = Uint64 s } ->
+                  (match server.server_nfiles with
+		  | None -> server.server_nfiles <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "maxusers" ; tag_value = Uint64 s } ->
+                  (match server.server_max_users with
+		  | None -> server.server_max_users <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "softfiles" ; tag_value = Uint64 s } ->
+                  (match server.server_soft_limit with
+		  | None -> server.server_soft_limit <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "hardfiles" ; tag_value = Uint64 s } ->
+                  (match server.server_hard_limit with
+		  | None -> server.server_hard_limit <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "auxportslist" ; tag_value = String s } ->
+                  server.server_auxportslist <- s
+              |  { tag_name = Field_UNKNOWN "lowusers" ; tag_value = Uint64 s } ->
+                  (match server.server_lowid_users with
+		  | None -> server.server_lowid_users <- Some s | _ -> ())
+              |  { tag_name = Field_UNKNOWN "tcpportobfuscation" ; tag_value = Uint64 s } ->
+                  server.server_obfuscation_tcp <- Some (Int64.to_int s)
+              |  { tag_name = Field_UNKNOWN "udpportobfuscation" ; tag_value = Uint64 s } ->
+                  server.server_obfuscation_udp <- Some (Int64.to_int s)
+              |  { tag_name = Field_UNKNOWN "country" ; tag_value = String s } -> ()
+              |  { tag_name = Field_UNKNOWN "udpflags" ; tag_value = Uint64  s } -> ()
+              |  { tag_name = Field_UNKNOWN "refs" ; tag_value = Uint64  s } -> ()
+              | _ -> lprintf_nl "parsing server.met, unknown field %s" (string_of_tag tag)
+          ) r.S.tags;
+	  server_must_update server
         with _ -> ()
     ) ss;
     List.length ss
@@ -125,16 +174,16 @@ let unpack_server_met filename url =
 	      Unix2.tryopen_read_zip filename (fun ic ->
 		try
 		  let file = Zip.find_entry ic "server.met" in
-      lprintf_nl "server.met found in %s" url;
+		    lprintf_nl (_b "server.met found in %s") url;
 		  file.Zip.filename
 		with e ->
-      lprintf_nl "Exception %s while extracting server.met from %s"
+		    lprintf_nl (_b "Exception %s while extracting server.met from %s")
 		    (Printexc2.to_string e) url;
 		  raise e) in
 	    (try
 	      ignore(Misc.archive_extract filename "zip")
 	    with e ->
-        lprintf_nl "Exception %s while extracting server.met from %s"
+		lprintf_nl (_b "Exception %s while extracting server.met from %s")
 		(Printexc2.to_string e) url;
 	      raise e);
 	    result
@@ -170,20 +219,17 @@ let download_server_met url =
         let s = unpack_server_met filename url in
     let n = load_server_met s in
       if s <> filename then Sys.remove s;
-            lprintf_nl "server.met loaded from %s, %d servers found, %d new ones inserted"
+            lprintf_nl (_b "server.met loaded from %s, %d servers found, %d new ones inserted")
         url n ((List.length (Hashtbl2.to_list servers_by_key)) - nservers)
       with e -> ()
     )
 
-let already_done = Failure "File already downloaded (use 'force_download' if necessary)"
+let already_done = Failure (Printf.sprintf (_b "File already downloaded (use 'force_download' if necessary)"))
+let no_download_to_force = Failure (Printf.sprintf (_b "No forceable download found"))
+exception Already_downloading of string
+exception Already_shared of string
 
-let no_download_to_force = Failure "No forceable download found"
-
-let already_downloading = Failure "File is already in download queue"
-
-let already_shared = Failure "File is already shared"
-
-let really_query_download filenames size md4 location old_file absents =
+let really_query_download filename size md4 location old_file absents user =
 
   begin
     try
@@ -221,15 +267,7 @@ let really_query_download filenames size md4 location old_file absents =
   end;
 
 (* TODO RESULT  let other_names = DonkeyIndexer.find_names md4 in *)
-  let filenames =
-    let other_names = [] in
-    let filenames = List.fold_left (fun names name ->
-          if List.mem name names then names else names @ [name]
-      ) filenames other_names in
-    (List.map (fun name -> name , { ips=[]; nips=0}) filenames)
-  in
-
-  let file = new_file file_diskname FileDownloading md4 size filenames true in
+  let file = new_file file_diskname FileDownloading md4 size filename true user in
   begin
     match absents with
       None -> ()
@@ -241,7 +279,7 @@ let really_query_download filenames size md4 location old_file absents =
             CommonSwarming.set_absent swarmer absents
   end;
 
-  if !verbose then lprintf_nl "Started new download, file %s, size %Ld, md4 %s"
+  if !verbose then lprintf_nl (_b "Started new download, file %s, size %Ld, md4 %s")
     (file_best_name file) size (Md4.to_string md4);
 
   DonkeyProtoOvernet.Overnet.recover_file file;
@@ -298,7 +336,7 @@ with _ -> ()
   );
   as_file file
 
-let query_download filenames size md4 location old_file absents force =
+let query_download filename size md4 location old_file absents force user =
   if force then
     if !forceable_download = [] then
       raise no_download_to_force
@@ -306,20 +344,23 @@ let query_download filenames size md4 location old_file absents force =
       begin
         let f = List.hd !forceable_download in
 	  forceable_download := [];
-          really_query_download f.result_names f.result_size md4 None None None
+	  really_query_download (List.hd f.result_names) f.result_size md4 None None None user
       end
   else
     begin
       try
         let file = find_file md4 in
 	  if (file_state file) = FileShared then
-	    raise already_shared
+	    raise (Already_shared (Printf.sprintf (_b "File is already shared%s")
+	      (match file.file_shared with
+		 None -> ""
+	       | Some sh -> (" in " ^ (Filename2.dirname sh.impl_shared_fullname)))))
 	  else
 	    begin
 (* jave TODO: if a user currently not downloading this file is requesting the download add this user
    to the list of users currently downloading this file *)
 	      forceable_download := [];
-	      raise already_downloading 
+	      raise (Already_downloading (Printf.sprintf (_b "File is already in download queue of %s") (file_owner (as_file file)).CommonTypes.user_name))
 	    end
       with Not_found ->
         begin
@@ -327,7 +368,7 @@ let query_download filenames size md4 location old_file absents force =
           (* copy file info into result for later usage in force_download *)
           let r = { dummy_result with
 	      result_uids = [Uid.create (Ed2k md4)];
-              result_names = filenames;
+              result_names = [filename];
               result_size = size;
               result_force = true; (* marker for force_download *)
 	      result_modified = false;
@@ -339,19 +380,19 @@ let query_download filenames size md4 location old_file absents force =
         else
           begin
             forceable_download := [];
-            really_query_download filenames size md4 location old_file absents
+            really_query_download filename size md4 location old_file absents user
           end
         end
     end
 
-let result_download r filenames force =
+let result_download r filenames force user =
   let rec iter uids =
     match uids with
       [] -> raise IgnoreNetwork
     | uid :: tail ->
         match Uid.to_uid uid with
           Ed2k md4 ->
-            query_download filenames r.result_size md4 None None None force
+            query_download (List.hd filenames) r.result_size md4 None None None force user
         | _  -> iter tail
   in
   iter r.result_uids
@@ -378,20 +419,23 @@ let import_temp temp_dir =
           if Sys.file_exists met then
             let s = File.to_string met in
             let f = P.read s in
-            let filenames = ref [] in
+            let filename_met = ref None in
             let size = ref Int64.zero in
             List.iter (fun tag ->
                 match tag with
                   { tag_name = Field_Filename; tag_value = String s } ->
                     lprintf_nl "Import Donkey %s" s;
 
-                    filenames := s :: !filenames;
+                    filename_met := Some s;
                 | { tag_name = Field_Size; tag_value = Uint64 v } ->
                     size := v
                 | _ -> ()
             ) f.P.tags;
-            ignore (really_query_download !filenames !size f.P.md4 None
-              (Some filename) (Some (List.rev f.P.absents)));
+            ignore (really_query_download
+		(match !filename_met with
+		   None -> filename
+		| Some s -> s) !size f.P.md4 None
+              (Some filename) (Some (List.rev f.P.absents)) CommonUserDb.admin_user);
       with _ -> ()
   ) list
 
@@ -508,7 +552,7 @@ let recover_md4s md4 =
 
 
 
-let parse_donkey_url url =
+let parse_donkey_url url user =
   let url = Str.global_replace (Str.regexp "|sources,") "|sources|" url in
   match String2.split url '|' with
 (* TODO RESULT *)
@@ -516,7 +560,7 @@ let parse_donkey_url url =
   | "file" :: name :: size :: md4 :: "/" :: "sources" :: sources :: _ ->
 (*  ed2k://|file|Wikipedia_3.3_noimages.iso|2666311680|747735CD46B61DA92973E9A8840A9C99|/|sources,62.143.4.124:4662|/  *)
       if Int64.of_string size >= 4294967295L then
-	"Files > 4GB are not allowed", false
+	(Printf.sprintf (_b "Files > 4GB are not allowed")), false
       else
         begin
 	  let md4 = if String.length md4 > 32 then
@@ -535,8 +579,8 @@ let parse_donkey_url url =
           end) s;
           begin
 	    try
-              let file = query_download [name] (Int64.of_string size)
-                (Md4.of_string md4) None None None false in
+              let file = query_download name (Int64.of_string size)
+                (Md4.of_string md4) None None None false user in
 	      let new_file = find_file (Md4.of_string md4) in
 	      CommonInteractive.start_download file;
 	      if !new_sources <> [] then
@@ -544,34 +588,39 @@ let parse_donkey_url url =
 	          List.iter (fun (source_ip, source_port) ->
     		    add_source new_file source_ip source_port Ip.null 0
     	          ) !new_sources;
-    	          (Printf.sprintf "added %d sources to new download" (List.length !new_sources)), true
+    	          (Printf.sprintf (_b "added %d sources to new download") (List.length !new_sources)), true
     	        end
     	      else "", true
-    	    with e -> (Printexc2.to_string e), false
+    	    with
+	      Already_downloading (s)
+	    | Already_shared (s) -> s, false
+	    | e -> (Printexc2.to_string e), false
     	  end
     	end
   | "ed2k://" :: "file" :: name :: size :: md4 :: _
   | "file" :: name :: size :: md4 :: _ ->
       if Int64.of_string size >= 4294967295L then
-	"Files > 4GB are not allowed", false
+	(Printf.sprintf (_b "Files > 4GB are not allowed")), false
       else
         let md4 = if String.length md4 > 32 then
           String.sub md4 0 32 else md4 in
 	let name =
-	  let name2 = Filename2.filesystem_compliant name "" in
+	  let name2 = Filename2.filesystem_compliant name "" 0 in
 	    if name2 = "" then
 	      Printf.sprintf "urn_ed2k_%s" md4
 	    else
 	      name2
 	in
           begin try
-            let file = query_download [name] (Int64.of_string size)
-              (Md4.of_string md4) None None None false;
+            let file = query_download name (Int64.of_string size)
+              (Md4.of_string md4) None None None false user;
             in
             CommonInteractive.start_download file;
             "", true
-          with e ->
-            (Printexc2.to_string e), false
+          with
+	    Already_downloading (s)
+	  | Already_shared (s) -> s, false
+	  | e -> (Printexc2.to_string e), false
           end
   | "ed2k://" :: "server" :: ip :: port :: _
   | "server" :: ip :: port :: _ ->
@@ -594,6 +643,10 @@ let parse_donkey_url url =
 
   | _ -> "", false
 
+let ip_of_server_cid s =
+  match s.server_cid with
+    None -> Ip.null
+  | Some ip -> ip
 
 let op_file_check file =
   match file.file_swarmer with
@@ -695,7 +748,7 @@ let commands = [
         end
       else
         if Sys.file_exists filename then begin
-                let n = load_server_met filename in
+                let n = load_server_met (unpack_server_met filename "") in
                   Printf.sprintf "%d servers found, %d new ones inserted"
         n ((List.length (Hashtbl2.to_list servers_by_key)) - nservers)
           end
@@ -703,21 +756,21 @@ let commands = [
           Printf.sprintf "%s does not exist, ignoring..." filename
     end
   else
-    Printf.sprintf "ED2K_update_server_list_met is disabled, ignoring..."
+    Printf.sprintf "ED2K-update_server_list_server_met is disabled, ignoring..."
     ), "<filename|URL> :\t\tadd the servers from a server.met file or URL";
 
     "id", Arg_none (fun o ->
         let buf = o.conn_buf in
         List.iter (fun s ->
-            Printf.bprintf buf "For %s:%d  --->   %s\n"
-              (Ip.to_string s.server_ip) s.server_port
+            Printf.bprintf buf "For %s (%s) --->   %s\n"
+              (string_of_server s) s.server_name
               (match s.server_cid with
                 None -> "waiting"
               | Some ip ->
-                  if Ip.valid ip then
-                    Ip.to_string ip
+                  if low_id ip then
+                    Printf.sprintf "%s (LowID)" (Int64.to_string (Ip.to_int64 (Ip.rev ip)))
                   else
-                    Int64.to_string (Ip.to_int64 (Ip.rev ip)))
+                    Printf.sprintf "%s (HighID)" (Ip.to_string (ip_of_server_cid s)))
         ) (connected_servers());
         ""
     ), ":\t\t\t\t\tprint ID on connected servers";
@@ -760,11 +813,11 @@ let commands = [
 
     "bs", Arg_multiple (fun args o ->
         List.iter (fun arg ->
-            let ip = Ip.of_string arg in
-            server_black_list =:=  ip :: !!server_black_list;
+            let range = Ip.range_of_string arg in
+            server_black_list =:=  range :: !!server_black_list;
         ) args;
         "done"
-    ), "<ip1> <ip2> ... :\t\t\tadd these IPs to the servers black list";
+    ), "<range1> <range2> ... :\t\t\tadd these IPs to the servers black list (can be single IPs, CIDR ranges or begin-end ranges)";
 
     "port", Arg_one (fun arg o ->
         donkey_port =:= int_of_string arg;
@@ -772,6 +825,7 @@ let commands = [
     "<port> :\t\t\t\tchange connection port";
 
     "scan_temp", Arg_none (fun o ->
+	if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
         let buf = o.conn_buf in
         let list = Unix2.list_directory !!temp_directory in
 
@@ -831,7 +885,7 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
         \\<input style=\\\"font: 8pt sans-serif\\\" name=\\\"newName\\\" type=text size=50 value=\\\"%s\\\"\\>\\</input\\>\\</td\\>\\</form\\>
                 \\<td class=\\\"sr \\\"\\>%s\\</td\\>
                 \\<td class=\\\"sr \\\"\\>\\<A HREF=\\\"%s\\\"\\>%s\\</A\\>\\</td\\>\\</tr\\>"
-                    !tr fnum fnum fnum (file_best_name file) "Downloading" (file_comment (as_file file)) filename
+                    !tr fnum fnum fnum (file_best_name file) "Downloading" (file_print_ed2k_link (file_best_name file) (file_size file) file.file_md4) filename
                 else
                   Printf.bprintf buf "%s is %s %s\n" filename
                     (file_best_name file)
@@ -869,24 +923,22 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
         ) list;
 
         if use_html_mods o then Printf.bprintf buf "\\</table\\>\\</div\\>";
+        "" end
+	else begin
+	  print_command_result o o.conn_buf "You are not allowed to use scan_temp";
+	"" end
 
-        ""
     ), ":\t\t\t\tprint temp directory content";
 
     "sources", Arg_none (fun o ->
-        let buf = o.conn_buf in
-        DonkeySources.print buf o.conn_output;
-        ""
+	if CommonUserDb.user2_is_admin o.conn_user.ui_user then begin
+          DonkeySources.print o.conn_buf o.conn_output;
+          "" end
+	else begin
+	  print_command_result o o.conn_buf "You are not allowed to list sources";
+	  "" end
     ), ":\t\t\t\tshow sources currently known";
 
-    (*
-    "update_sources", Arg_none (fun o ->
-        let buf = o.conn_buf in
-        DonkeySources.recompute_ready_sources ();
-        "done"
-    ), ":\t\t\trecompute order of connections to sources (experimental)";
-*)
-
     "xs", Arg_none (fun o ->
         let buf = o.conn_buf in
         if !xs_last_search >= 0 then begin
@@ -915,8 +967,8 @@ parent.fstatus.location.href='submit?q=rename+'+i+'+\\\"'+encodeURIComponent(for
 
 (* TODO RESULT *)
     "dd", Arg_two(fun size md4 o ->
-        let file = query_download [] (Int64.of_string size)
-          (Md4.of_string md4) None None None false in
+        let file = query_download md4 (Int64.of_string size)
+          (Md4.of_string md4) None None None false o.conn_user.ui_user in
         CommonInteractive.start_download file;
         "download started"
     ), "<size> <md4> :\t\t\tdownload from size and md4";
@@ -997,12 +1049,10 @@ file.---------> to be done urgently
         | Connection sock ->
             client_send c (DonkeyProtoClient.SayReq s)
       with
-        Not_found ->
-          CommonChat.send_text !!CommonOptions.chat_console_id None
-            (Printf.sprintf "client %s unknown" iddest)
+        Not_found -> ()
   );
-  network.op_network_download <- (fun r ->
-      result_download r r.result_names r.result_force
+  network.op_network_download <- (fun r user ->
+      result_download r r.result_names r.result_force user
   )
 
 module P = GuiTypes
@@ -1023,7 +1073,6 @@ let _ =
             { (impl_file_info file.file_file) with
 
             P.file_network = network.network_num;
-            P.file_names = file.file_filenames;
             P.file_md4 = file.file_md4;
             P.file_all_sources = file.file_sources.DonkeySources.manager_all_sources;
             P.file_active_sources = file.file_sources.DonkeySources.manager_active_sources;
@@ -1041,6 +1090,7 @@ let _ =
             P.file_format = file.file_format;
             P.file_chunks_age = last_seen;
             P.file_uids = [Uid.create (Ed2k file.file_md4)];
+            P.file_comments = file.file_comments
           } in
         v
       with e ->
@@ -1059,19 +1109,37 @@ let _ =
           P.server_port = s.server_port;
           P.server_realport = (match s.server_realport with Some x -> x | None -> 0);
           P.server_score = s.server_score;
-          P.server_nusers = s.server_nusers;
-          P.server_nfiles = s.server_nfiles;
+          P.server_nusers = (match s.server_nusers with None -> 0L | Some v -> v);
+          P.server_nfiles = (match s.server_nfiles with None -> 0L | Some v -> v);
           P.server_name = s.server_name;
           P.server_description = s.server_description;
           P.server_banner = s.server_banner;
           P.server_preferred = s.server_preferred;
-	  P.server_version = s.server_version;
-          P.server_max_users = s.server_max_users;
-          P.server_soft_limit = s.server_soft_limit;
-          P.server_hard_limit = s.server_hard_limit;
-          P.server_lowid_users = s.server_lowid_users;
+          P.server_master = s.server_master;
+          P.server_published_files = (List.length s.server_sent_shared);
+          P.server_version = s.server_version;
+          P.server_max_users = (match s.server_max_users with None -> 0L | Some v -> v);
+          P.server_soft_limit = (match s.server_soft_limit with None -> 0L | Some v -> v);
+          P.server_hard_limit = (match s.server_hard_limit with None -> 0L | Some v -> v);
+          P.server_lowid_users = (match s.server_lowid_users with None -> 0L | Some v -> v);
           P.server_ping = s.server_ping;
-
+          P.server_features = let temp_buf = Buffer.create 100 in
+            if s.server_has_zlib then Printf.bprintf temp_buf "zlib ";
+            if s.server_has_newtags then Printf.bprintf temp_buf "newtags ";
+            if s.server_has_unicode then Printf.bprintf temp_buf "unicode ";
+            if s.server_has_related_search then Printf.bprintf temp_buf "related_search ";
+            if s.server_has_tag_integer then Printf.bprintf temp_buf "tag_integer ";
+            if s.server_has_largefiles then Printf.bprintf temp_buf "largefiles ";
+            if s.server_has_get_sources then Printf.bprintf temp_buf "getsources ";
+            if s.server_has_get_sources2 then Printf.bprintf temp_buf "getsources2 ";
+            if s.server_has_get_files then Printf.bprintf temp_buf "getfiles ";
+            (match s.server_obfuscation_tcp with
+             | Some p when p <> 0 -> Printf.bprintf temp_buf "tcp_obfuscation(%d) " p | _ -> ());
+            (match s.server_obfuscation_udp with
+             | Some p when p <> 0 -> Printf.bprintf temp_buf "udp_obfuscation(%d) " p | _ -> ());
+            if s.server_auxportslist <> "" then Printf.bprintf temp_buf "auxportslist %s " s.server_auxportslist;
+            if s.server_dynip <> "" then Printf.bprintf temp_buf "dynip %s " s.server_dynip;
+            if Buffer.contents temp_buf <> "" then Some (Buffer.contents temp_buf) else None;
         }
       else raise Not_found
   )
@@ -1117,21 +1185,22 @@ let _ =
             Direct_address (ip, port) -> Known_location (ip,port)
 	  | Indirect_address (server_ip, server_port, ip, port, real_ip) -> 
 	      Indirect_location (c.client_name,c.client_md4, real_ip, port)
-          | _ -> Indirect_location (c.client_name,c.client_md4, Ip.null, 0));
+          | _ -> Indirect_location (c.client_name,c.client_md4, c.client_ip, 0));
         P.client_state = client_state c;
         P.client_type = client_type c;
         P.client_name = c.client_name;
         P.client_rating = c.client_rating;
         P.client_connect_time = c.client_connect_time;
         P.client_software = brand_to_string_short c.client_brand;
+        P.client_os = c.client_osinfo;
         P.client_release = c.client_emule_proto.emule_release;
         P.client_emulemod = brand_mod_to_string_short c.client_brand_mod;
         P.client_downloaded = c.client_downloaded;
         P.client_uploaded = c.client_uploaded;
 (*        P.client_source.source_sock_addr =    (); *)
         P.client_upload =
-        (match c.client_upload with
-            Some cu -> Some (file_best_name cu.up_file)
+        (match client_upload (as_client c) with
+            Some f -> Some (CommonFile.file_best_name f)
           | None -> None);
         P.client_sui_verified = c.client_sui_verified;
       }
@@ -1139,11 +1208,6 @@ let _ =
   client_ops.op_client_debug <- (fun c debug ->
       c.client_debug <- debug)
 
-let ip_of_server_cid s =
-  match s.server_cid with
-    None -> Ip.null
-  | Some ip -> ip
-
 let _ =
   server_ops.op_server_remove <- (fun s ->
       DonkeyGlobals.remove_server s.server_ip s.server_port
@@ -1169,6 +1233,9 @@ let _ =
   server_ops.op_server_users <- (fun s ->
       List2.tail_map (fun u -> as_user u.user_user) s.server_users)    ;
 
+  server_ops.op_server_published <- (fun s ->
+      List.map (fun f -> as_file f) s.server_sent_shared);
+
   server_ops.op_server_cid <- (fun s -> ip_of_server_cid s);
 
   server_ops.op_server_low_id <- (fun s -> low_id (ip_of_server_cid s));
@@ -1185,8 +1252,13 @@ let _ =
 
 let _ =
   file_ops.op_file_save_as <- (fun file name ->
-      file.file_filenames <- [name, noips()];
-      set_file_best_name (as_file file) name
+      add_file_filenames (as_file file) name;
+      set_file_best_name (as_file file) name "" 0
+  );
+  file_ops.op_file_shared <- (fun file ->
+      match file.file_shared with
+	None -> None
+      | Some sh -> Some (as_shared sh)
   );
   file_ops.op_file_set_format <- (fun file format ->
       file.file_format <- format);
@@ -1212,10 +1284,16 @@ let _ =
       ) file.file_sources;
       !list
   );
-  file_ops.op_file_print_html <- (fun file buf ->
-
-     html_mods_cntr_init ();
+  file_ops.op_file_print <- (fun file o ->
 
+     let buf = o.conn_buf in
+     if not (use_html_mods o) then begin
+	let cntr = ref 0 in
+	List.iter (fun (ip, n, r, c) ->
+	  incr cntr;
+	  Printf.bprintf buf
+	    "Comment %d: Rating(%d): %s (%s/%s)\n" !cntr r (Charset.to_utf8 c) n (Ip.to_string ip)) file.file_comments
+     end else begin
      let tr () =
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ())
      in
@@ -1227,32 +1305,35 @@ let _ =
             (Md4.to_string file.file_md4) (Md4.to_string file.file_md4)
         ) ];
 
+      let cntr = ref 0 in
+      List.iter (fun (ip, n, r, c) -> 
+        incr cntr;
+        tr ();
+        html_mods_td buf [
+          ("Comment", "sr br", Printf.sprintf "Comment %d" !cntr);
+          ("User rating and comment", "sr", Printf.sprintf "Rating(%d): %s (%s/%s)" r (Charset.to_utf8 c) n (Ip.to_string ip));
+        ];
+      ) file.file_comments;
+
       tr ();
       html_mods_td buf [
         ("File History Links", "sr br", "File History");
         ("","sr", Printf.sprintf
-"\\<a target=\\\"_blank\\\" href=\\\"http://62.241.53.2/cgi-bin/ed2khistory?ed2k=%s\\\"\\>DonkeyServer 1 File History\\</a\\>
- \\<a target=\\\"_blank\\\" href=\\\"http://tothbenedek.hu/ed2kstats/ed2k?hash=%s\\\"\\>Toth File History\\</a\\>
- \\<a target=\\\"_blank\\\" href=\\\"http://ed2k.titanesel.ws/ed2k.php?hash=%s\\\"\\>Titanesel File History\\</a\\>"
-            (Md4.to_string file.file_md4) (Md4.to_string file.file_md4) (Md4.to_string file.file_md4)
+"\\<a target=\\\"_blank\\\" href=\\\"http://tothbenedek.hu/ed2kstats/ed2k?hash=%s\\\"\\>Toth File History\\</a\\>
+\\<a target=\\\"_blank\\\" href=\\\"http://ed2k.titanesel.ws/ed2k.php?hash=%s\\\"\\>Titanesel File History\\</a\\>"
+            (Md4.to_string file.file_md4) (Md4.to_string file.file_md4)
 	)
        ];
       tr ();
+      let ed2k = file_print_ed2k_link (file_best_name file) (file_size file) file.file_md4 in
       html_mods_td buf [
         ("ed2k link", "sr br", "ed2k link");
-        ("", "sr", Printf.sprintf "\\<a href=\\\"ed2k://|file|%s|%s|%s|/\\\"\\>ed2k://|file|%s|%s|%s|/\\</A\\>"
-            (Url.encode (file_best_name file))
-          (Int64.to_string (file_size file))
-          (Md4.to_string file.file_md4)
-          (file_best_name file)
-          (Int64.to_string (file_size file))
-          (Md4.to_string file.file_md4)) ];
-
+        ("", "sr", Printf.sprintf "\\<a href=\\\"%s\\\"\\>%s\\</A\\>" ed2k ed2k) ];
       tr ();
       let optionlist = ref "" in
-      List.iter (fun (name,_) ->
+      List.iter (fun name ->
           optionlist := !optionlist ^ Printf.sprintf "\\<option value=\\\"%s\\\"\\>%s\n" name name;
-      ) file.file_filenames;
+      ) file.file_file.impl_file_filenames;
 
 
       let input_size = (Printf.sprintf "%d" ((String.length (file_best_name file))+3)) in
@@ -1268,19 +1349,22 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
 //--\\>
 \\</script\\>" (file_num file)
 
-   ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>"
+   ^ "\\<table border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>"
    ^ "\\<form name=\\\"renameForm1\\\" id=\\\"renameForm1\\\" action=\\\"javascript:submitRenameForm(1);\\\"\\>"
+   ^ "\\<td\\>"
    ^ "\\<select name=\\\"newName\\\" id=\\\"newName\\\" onchange=\\\"javascript:renameForm2.newName.value=renameForm1.newName.options[renameForm1.newName.selectedIndex].value;this.form.submit();\\\"\\>"
    ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\n" (file_best_name file) (file_best_name file)
    ^ !optionlist
-   ^ "\\</select\\>\\</td\\>\\</tr\\>\\</form\\>\\<tr\\>\\<td\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
-   ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</tr\\>\\</form\\>\\</table\\>"
+   ^ "\\</select\\>\\</td\\>\\</form\\>\\</tr\\>\\<tr\\>\\<form name=\\\"renameForm2\\\" id=\\\"renameForm2\\\" action=\\\"javascript:submitRenameForm(2);\\\"\\>"
+   ^ "\\<td\\>"
+   ^ "\\<input name=\\\"newName\\\" type=text size=" ^ input_size ^ " value=\\\"" ^ (file_best_name file) ^ "\\\"\\>\\</input\\>\\</td\\>\\</form\\>\\</tr\\>\\</table\\>"
     ) ];
-
-
+  end
   );
-  file_ops.op_file_print_sources_html <- (fun file buf ->
+  file_ops.op_file_print_sources <- (fun file o ->
 
+    if not (use_html_mods o) then raise Not_found;
+    let buf = o.conn_buf in
     let sources_list = ref [] in
     DonkeySources.iter_relevant_sources (fun s ->
       let s_uid = s.DonkeySources.source_uid in
@@ -1373,7 +1457,8 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
             ((string_of_connection_state (client_state c)), "sr",
               (short_string_of_connection_state (client_state c)) );
             (String.escaped c.client_name, "sr", client_short_name c.client_name);
-            (brand_to_string c.client_brand, "sr", brand_to_string_short c.client_brand);
+            (client_software (brand_to_string c.client_brand) c.client_osinfo, "sr",
+	     client_software_short (brand_to_string_short c.client_brand) c.client_osinfo);
             ("", "sr", c.client_emule_proto.emule_release);
 
             ] @
@@ -1453,12 +1538,6 @@ parent.fstatus.location.href='submit?q=rename+%d+\\\"'+encodeURIComponent(formID
       DonkeyProtoOvernet.Overnet.cancel_recover_file file;
       DonkeyProtoKademlia.Kademlia.cancel_recover_file file;
   );
-  file_ops.op_file_comment <- (fun file ->
-      Printf.sprintf "ed2k://|file|%s|%Ld|%s|"
-        (Url.encode (file_best_name file))
-      (file_size file)
-      (Md4.to_string file.file_md4)
-  );
   file_ops.op_file_files <- (fun file impl ->
       match file.file_swarmer with
         None -> [CommonFile.as_file impl]
@@ -1472,16 +1551,10 @@ let try_recover_temp_file filename md4 =
       let file_diskname = Filename.concat !!temp_directory filename in
       let size = Unix32.getsize file_diskname in
       if size <> zero then
-        let names =
-            (* TODO RESULT
-            try DonkeyIndexer.find_names md4
-            with _ -> *)
-            []
-        in
-        let _ =
-          really_query_download names size md4 None (Some file_diskname) None
-        in
-        recover_md4s md4
+	begin
+	  ignore (really_query_download (Md4.to_string md4) size md4 None (Some file_diskname) None CommonUserDb.admin_user);
+	  recover_md4s md4
+	end
 
 let _ =
   network.op_network_extend_search <- (fun s e ->
@@ -1528,7 +1601,31 @@ let _ =
   );
 
   network.op_network_parse_url <- parse_donkey_url;
+  network.op_network_reset <- (fun _ -> ());
 
+  network.op_network_close_search <- (fun s -> ());
+  network.op_network_porttest_start <- (fun _ ->
+    porttest_result := PorttestInProgress (last_time ());
+    let module H = Http_client in
+    let r = { H.basic_request with
+      H.req_url = Url.of_string
+        (Printf.sprintf "http://porttest.emule-project.net:81/ct_noframe.php?lang=&tcpport=%d&udpport=%d"
+          !!donkey_port (!!donkey_port + 4));
+      H.req_proxy = !CommonOptions.http_proxy;
+      H.req_max_retry = 10;
+      H.req_user_agent = get_user_agent () } in
+    H.wget r (fun file ->
+      Unix2.tryopen_read file (fun cin ->
+	try
+	  while true do
+	    let line = input_line cin in
+	      try
+		if Str.string_match (Str.regexp "^<P>Testing IP") line 0 then
+		  porttest_result := PorttestResult (last_time (), line)
+	      with _ -> ()
+	  done
+	with End_of_file -> ())
+    ));
   network.op_network_forget_search <- forget_search
 
 (* emule<->mldonkey disconnects during chat, and this doesn't seem to auto reconnect
@@ -1559,7 +1656,9 @@ let _ =
             let module C = M.ViewFiles in
             M.ViewFilesReq C.t);
       | NoConnection ->
-	  if !verbose then lprintf_nl "re-connecting friend %s" (full_client_identifier c);
+	  if !verbose then lprintf_nl "re-connecting friend %s"
+	    (full_client_identifier c);
+	  set_must_browse (as_client c);
           reconnect_client c
       | _ -> ()
   );
@@ -1704,24 +1803,23 @@ let _ =
     (fun url filename ->
     if !!enable_donkey && !!update_server_list_server_met then
       begin
-        lprintf_n "server.met loaded from %s" url;
+        lprintf_nl "server.met loaded from %s" url;
   begin
     try
       let s = unpack_server_met filename url in
         let nservers = List.length (Hashtbl2.to_list servers_by_key) in
               let n = load_server_met s in
                 if s <> filename then Sys.remove s;
-            lprintf ", %d servers found, %d new ones inserted"
+            lprintf_nl "%d servers found, %d new ones inserted"
         n ((List.length (Hashtbl2.to_list servers_by_key)) - nservers)
            with _ -> ()
   end;
-        lprint_newline ()
       end
     else
       if not !!enable_donkey then
         lprintf_nl "eDonkey module is disabled, ignoring..."
       else
-        lprintf_nl "ED2K_update_server_list_met is disabled, ignoring..."
+        lprintf_nl "ED2K-update_server_list_server_met is disabled, ignoring..."
   );
   CommonWeb.add_web_kind "comments.met" "List of edonkey files comments" 
     (fun _ filename ->
