@@ -100,7 +100,7 @@ done;
                 end
 
 let remove_client_slot c =
-  set_client_has_a_slot (as_client c) false;
+  set_client_has_a_slot (as_client c) NoSlot;
   client_send c (
     let module M = DonkeyProtoClient in
     let module Q = M.CloseSlot in
@@ -111,8 +111,11 @@ let unshare_file file =
   match file.file_shared with
     None -> ()
   | Some s -> 
-      if !verbose_share then
-        lprintf_nl "unshare_file %s" file.file_diskname;
+      if !verbose_share || !verbose then
+        lprintf_nl "Unsharing file %s" (file_best_name file);
+      List.iter (fun s ->
+        s.server_sent_shared <- List2.removeq file s.server_sent_shared;
+      ) (connected_servers ());
       file.file_shared <- None;
       decr nshared_files;
       CommonShared.shared_calculate_total_bytes ();
@@ -269,7 +272,7 @@ let send_get_range_request c file ranges =
         | _ -> assert false
       in
       let msg = M.QueryBlocReq msg in
-      set_read_power sock (!!upload_power + maxi 0 (file_priority file));
+      set_read_power sock (!!upload_power + max 0 (file_priority file));
 (*      lprintf "QUEUE DOWNLOAD REQUEST\n"; *)
 (*      CommonUploads.queue_download_request (fun _ ->  *)
           client_send c msg 
@@ -305,7 +308,7 @@ let rec get_from_client c =
   | Some (file,up) ->
       
       try
-        let _ = CommonSwarming.current_block up in
+        let _ = CommonSwarming.current_blocks up in
         let ranges = CommonSwarming.current_ranges up in
         let before_find_range = List.length ranges in
 
@@ -344,7 +347,7 @@ let rec get_from_client c =
           try
             let swarmer = CommonSwarming.uploader_swarmer up in       
             (try CommonSwarming.verify_one_chunk swarmer with _ -> ());
-            let _ = CommonSwarming.find_block up in
+            let _ = CommonSwarming.find_blocks up in
             get_from_client c
             
           with Not_found ->
@@ -435,7 +438,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
 			      (file_best_name file) (Printexc2.to_string e) in
                     Printf2.lprint_string m;
                     CommonEvent.add_event (Console_message_event m);
-                    file_pause (as_file file);
+                    file_pause (as_file file) CommonUserDb.admin_user;
                     raise e
                   end
           
