@@ -34,6 +34,7 @@ open CommonTypes
 open CommonOptions
 open CommonGlobals
 open CommonSources
+open CommonShared
 
 open DonkeyMftp
 open DonkeyImport
@@ -164,7 +165,7 @@ let udp_query_sources () =
                        > last_time () &&
                        s.server_next_udp <= last_time () then
                          begin
-                           if server_accept_multiple_getsources s then
+                           if s.server_has_get_sources then
                              new_servers := s :: !new_servers
                            else
                              old_servers := s :: !old_servers;
@@ -208,8 +209,36 @@ let udp_query_sources () =
   with e ->
     lprintf_nl "udp_query_sources: %s" (Printexc2.to_string e)
 
-
 let disconnect_server s reason =
+  let choose_new_master_server s =
+    match !DonkeyGlobals.master_server with
+    | Some ss when s == ss ->
+	DonkeyGlobals.master_server := None;
+        (try
+	  DonkeyGlobals.master_server :=
+	    Some (List.find (fun s -> s.server_master) !servers_list);
+	  if !verbose_location then begin
+            match !DonkeyGlobals.master_server with
+            | Some ns -> 
+	            lprintf_nl "changed main master server from %s (%s) to %s (%s)"
+	              ss.server_name (string_of_server ss) ns.server_name (string_of_server ns)
+            | _ -> ()
+          end
+        with Not_found -> ())
+    | _ -> ();
+  in
+(* remove this server from the list of servers a file was published to *)
+  let remove_server_from_shared_files s =
+    List.iter (fun file ->
+      shared_iter (fun sh ->
+        let impl = as_shared_impl sh in
+	  impl.impl_shared_servers <-
+	    List2.removeq (as_server s.server_server) impl.impl_shared_servers)
+    ) s.server_sent_shared;
+    s.server_sent_shared <- []
+  in
+  choose_new_master_server s;
+  remove_server_from_shared_files s;
   match s.server_sock with
       NoConnection -> ()
     | ConnectionWaiting token ->
@@ -223,18 +252,16 @@ let disconnect_server s reason =
         s.server_sock <- NoConnection;
         s.server_score <- s.server_score - 1;
         s.server_users <- [];
-        set_server_state s (NotConnected (reason, -1));
+        if server_state s = Connecting && String2.subcontains s.server_banner "This server is full" then
+          set_server_state s ServerFull
+        else
+          set_server_state s (NotConnected (reason, -1));
         s.server_master <- false;
-        (match !DonkeyGlobals.master_server with
-          | Some ss when s == ss ->
-              DonkeyGlobals.master_server := None;
-          | _ -> ());
         s.server_banner <- "";
         s.server_sent_all_queries <- false;
         remove_connecting_server s;
         remove_connected_server s
 
-
 let server_handler s sock event =
   match event with
       BASIC_EVENT (CLOSED r) ->
@@ -243,22 +270,61 @@ let server_handler s sock event =
         close sock Closed_for_timeout
     | _ -> ()
 
+let server_udp_ping_statreq s =
+  s.server_last_ping <- Unix.gettimeofday ();
+  s.server_failed_count <- s.server_failed_count + 1;
+  s.server_descping_counter <- s.server_descping_counter + 1;
+  if !verbose_msg_servers then
+    lprintf_nl "send UDP server ping to %s, try %d"
+      (string_of_server s) s.server_failed_count;
+  let module M = DonkeyProtoUdp in
+  let module E = M.ServerDescUdp in
+  let c = Int64.of_int (Random.int 65535) in
+  let challenge = 1437204480L ++ c in (* 0x55AA0000 *)
+  s.server_udp_ping_challenge <- Some challenge;
+(* next ping in 4,5h (16200s) - random value between 1s and 1h to avoid ping storms
+   eMule Changelog: August, 30. 2005
+   Unk: When a very large popular server come online,
+   it experienced ping storms every 4 hours.. Fixed (Lug) *)
+  s.server_next_ping <- s.server_last_ping +. 16200. -. (Random.float 3600.);
+  udp_server_send_ping s (M.PingServerUdpReq challenge);
+  if s.server_descping_counter < 2 then
+    begin
+      if !verbose_msg_servers then
+        lprintf_nl "send UDP server description request to %s" (string_of_server s);
+      let rand16 = Int64.of_int (Random.int 65535) in
+      let challenge = (left64 rand16 16) ++ M.ServerDescUdp.invalid_len in
+      s.server_udp_desc_challenge <- Some challenge;
+      udp_server_send s (M.ServerDescUdpReq challenge);
+    end
+  else
+    s.server_descping_counter <- 0
 
 let last_message_sender = ref (-1)
 
 let client_to_server s t sock =
   let module M = DonkeyProtoServer in
-
-  s.server_last_message <- last_time ();
+  s.server_failed_count <- 0;
 
   if !verbose_msg_servers then begin
-    lprintf_nl "Message from server:";
+    lprintf_nl "Message from server %s:" (string_of_server s);
     DonkeyProtoServer.print t; lprint_newline ()
   end;
 
   match t with
       M.SetIDReq t ->
         s.server_has_zlib <- t.M.SetID.zlib;
+        s.server_has_newtags <- t.M.SetID.newtags;
+        s.server_has_unicode <- t.M.SetID.unicode;
+        s.server_has_related_search <- t.M.SetID.related_search;
+        s.server_has_tag_integer <- t.M.SetID.tag_integer;
+        s.server_has_largefiles <- t.M.SetID.largefiles;
+        (match s.server_obfuscation_tcp with
+          | None -> if t.M.SetID.tcp_obfuscation then s.server_obfuscation_tcp <- Some 0
+          | Some p -> if not t.M.SetID.tcp_obfuscation then s.server_obfuscation_tcp <- None);
+        (match s.server_obfuscation_udp with
+          | None -> if t.M.SetID.udp_obfuscation then s.server_obfuscation_udp <- Some 0
+          | Some p -> if not t.M.SetID.udp_obfuscation then s.server_obfuscation_udp <- None);
         if low_id t.M.SetID.ip && !!force_high_id then
           disconnect_server s (Closed_for_error "Low ID")
         else begin
@@ -266,48 +332,57 @@ let client_to_server s t sock =
           s.server_realport <- t.M.SetID.port;
           (* disconnect after (connected_server_timeout) seconds of silence *)
           set_rtimeout sock !!connected_server_timeout;
-          set_server_state s Connected_initiating;
           remove_connecting_server s;
           s.server_score <- s.server_score + 5;
           connection_ok (s.server_connection_control);
 
+          if !!update_server_list_server then
           server_send sock (
-            let module A = M.AckID in
-            M.AckIDReq A.t
+            let module Q = M.QueryServerList in
+            M.QueryServerListReq Q.t
           );
 
           if not (low_id t.M.SetID.ip) && !!use_server_ip then
             last_high_id := t.M.SetID.ip;
+
+          (* nice and ugly, but it doesn't require any new fields *)
+          set_server_state s (Connected
+          ( match s.server_cid with
+              Some t -> if low_id t then (-1) else (-2) 
+              | _ -> (-1)
+          ));
+
+          if s.server_next_ping = 0. then server_udp_ping_statreq s;
+
+          (* fill list with queries for the server *)
+          fill_query_queue s;
+          s.server_sent_all_queries <- false;
+          s.server_queries_credit <- 0
         end
 
   | M.MessageReq msg ->
-
-      (* This is stupid *)
-      if s.server_version = "" then begin
-        try
-          let prefix = String.sub msg 0 15 in
-          if prefix = "server version " then 
-            s.server_version <- String.sub msg 15 4
-        with _ -> ()
-      end;
-
+      if msg <> "" then begin
       if !last_message_sender <> server_num s then begin
-          let server_header = Printf.sprintf "\n+-- From server %s [%s:%d] ------"
-              s.server_name (Ip.to_string s.server_ip) s.server_port in
-          CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" server_header));
+          let server_header = Printf.sprintf "+-- From server %s [%s] ------"
+              s.server_name (string_of_server s) in
+          CommonEvent.add_event (Console_message_event (Printf.sprintf "\n%s\n" server_header));
           if !CommonOptions.verbose_msg_servers then
-      lprintf_nl "%s" server_header;
+	    lprintf_nl "%s" server_header;
           last_message_sender := server_num s
         end;
       s.server_banner <- s.server_banner ^ Printf.sprintf "%s\n" msg;
       let msg = Printf.sprintf "| %s" msg in
       CommonEvent.add_event (Console_message_event (Printf.sprintf "%s\n" msg));
       if !CommonOptions.verbose_msg_servers then
-        lprintf_nl "%s" msg
+        begin
+          lprintf "%s: %s" (string_of_server s) msg;
+          lprint_newline ()
+        end
+      end
 
   | M.ServerListReq l ->
       if !!update_server_list_server then begin
-        if !verbose_msg_servers then lprintf_nl "Received serverlist from server";
+        if !verbose_msg_servers then lprintf_nl "Received %d servers from server" (List.length l);
         let module Q = M.ServerList in
         List.iter (fun s ->
             safe_add_server s.Q.ip s.Q.port
@@ -328,31 +403,16 @@ let client_to_server s t sock =
                 s.server_name <- name
             | { tag_name = Field_UNKNOWN "description"; tag_value = String desc } ->
                 s.server_description <- desc
-            | _ -> ()
-      ) s.server_tags;
-
-      (* nice and ugly, but it doesn't require any new fields *)
-      set_server_state s (Connected
-          ( match s.server_cid with
-              Some t -> if low_id t then (-1) else (-2) 
-              | _ -> (-1)
-          )
-        );
-
-      (* fill list with queries for the server *)
-      fill_query_queue s;
-      s.server_sent_all_queries <- false;
-      s.server_queries_credit <-  0
+            | _ -> lprintf_nl "parsing donkeyServers.ServerInfo, unknown field %s" (string_of_tag tag)
+      ) s.server_tags
 
   | M.InfoReq (users, files) ->
-      s.server_nusers <- Int64.of_int users;
-      s.server_nfiles <- Int64.of_int files;
+      s.server_nusers <- Some (Int64.of_int users);
+      s.server_nfiles <- Some (Int64.of_int files);
       if (users < !!min_users_on_server && not s.server_preferred) then
         begin
-          lprintf_nl "%s:%d remove server min_users_on_server limit hit!"
-            (Ip.to_string s.server_ip) s.server_port;
-
-          disconnect_server s Closed_for_timeout;
+          lprintf_nl "%s remove server min_users_on_server limit hit!"
+            (string_of_server s);
           server_remove (as_server s.server_server);
         end;
       server_must_update s
@@ -534,7 +594,7 @@ let rec connect_one_server restart =
                 lprintf_nl "Please import servers from a server.met file.";
                 lprintf_nl "Let MLDonkey use a file configured in web_infos";
                 lprintf_nl "or enter this link into MLDonkey:";
-    lprintf_nl "ed2k://|serverlist|http://www.jd2k.com/server.met|/"
+	        lprintf_nl "ed2k://|serverlist|http://www.gruk.org/server.met.gz|/"
               end;
 
               raise Not_found;
@@ -628,8 +688,7 @@ position to the min_left_servers position.
         if ls < min_last_conn && s.server_sock = NoConnection
           && not s.server_preferred then begin
           if !verbose then
-            lprintf_nl "old servers: Server too old: %s:%d"
-              (Ip.to_string s.server_ip) s.server_port;
+            lprintf_nl "old servers: Server too old: %s" (string_of_server s);
 
           to_remove := s :: !to_remove
         end
@@ -648,8 +707,6 @@ position to the min_left_servers position.
 
 (* Keep connecting to servers in the background. Don't stay connected to
   them , and don't send your shared files list *)
-let walker_list = ref []
-let delayed_list = ref []
 let next_walker_start = ref 0
 
 (* one call every 5 seconds, so 12/minute, 720/hour *)
@@ -684,55 +741,39 @@ let walker_timer () =
                   if connection_can_try s.server_connection_control then
                     begin
                       if !verbose then
-                        lprintf_nl "WALKER: try connect %s"
-                          (Ip.to_string s.server_ip);
+                        lprintf_nl "WALKER: try connect %s" (string_of_server s);
                       connect_server s
                     end
                   else
                     begin
                       delayed_list := s :: !delayed_list;
                       if !verbose then
-                        lprintf_nl "WALKER: connect %s delayed"
-                          (Ip.to_string s.server_ip);
+                        lprintf_nl "WALKER: connect %s delayed" (string_of_server s);
                     end
               | _ -> ()
 
-(* Keep connecting to servers in the background. Don't stay connected to
-  them , and don't send your shared files list *)
-let udp_walker_list = ref []
-let next_udp_walker_start = ref 0
-
-(* one call every second, so 3600/hour, must wait one hour before
-restarting
-Each client issues 1 packet/4hour, so 100000 clients means 25000/hour,
-7 packets/second = 7 * 40 bytes = 280 B/s ...
-*)
-
-
-let udp_walker_timer () =
-  match !udp_walker_list with
-    [] ->
-      if last_time () > !next_udp_walker_start then begin
-          next_udp_walker_start := last_time () + 4*3600;
-          Hashtbl.iter (fun _ s ->
-              udp_walker_list := s :: !udp_walker_list
-          ) servers_by_key;
+let udp_walker_timer () = (* called every 5s *)
+  if !!enable_servers then
+  let now = Unix.gettimeofday () in
+  try
+    Hashtbl.iter (fun _ s ->
+      if not (server_blocked (as_server s.server_server)) then
+        begin
+          if s.server_failed_count > 9 then
+            begin
+              lprintf_nl "remove dead server %s after %d not answered UDP pings"
+                (string_of_server s) s.server_failed_count;
+              server_remove (as_server s.server_server);
+              raise Exit
+            end;
+          if s.server_next_ping < now then
+            begin
+              server_udp_ping_statreq s;
+              raise Exit
+            end
         end
-  | s :: tail ->
-      udp_walker_list := tail;
-      let c = Int64.of_int (Random.int 65535) in
-      let challenge = 1437204480L ++ c in (* 0x55AA0000 *)
-      s.server_last_ping <- Unix.gettimeofday ();
-      udp_server_send_ping s (DonkeyProtoUdp.PingServerUdpReq challenge);
-
-      let module M = DonkeyProtoUdp in
-      let module E = M.ServerDescUdp in
-      udp_server_send s (M.ServerDescUdpReq {
-        E.ip = Ip.null;
-      })
-
-(*      UdpSocket.write (get_udp_sock ())
-                  true udp_ping s.server_ip (s.server_port + 4) *)
+    ) servers_by_key
+  with Exit -> ()
 
 (* sort the servers by preferred first
    then users count with decreasing order
@@ -740,7 +781,9 @@ let udp_walker_timer () =
 let compare_servers s2 s1 =
   let n = compare s1.server_preferred s2.server_preferred in
   if n = 0 then
-    Int64.to_int (s1.server_nusers -- s2.server_nusers)
+    Int64.to_int
+    ((match s1.server_nusers with None -> 0L | Some v -> v) --
+     (match s2.server_nusers with None -> 0L | Some v -> v))
   else n
 
 (* check connected servers *)
@@ -757,13 +800,13 @@ let update_master_servers _ =
         match s.server_sock with
         | Connection _ ->
             if !verbose_location then begin
-        if !tag2 then begin
+	      if !tag2 then begin
                 lprintf_n "master servers (old):";
-    tag1 := false;
-    tag2 := false
-        end;
-        lprintf " %s" (Ip.to_string s.server_ip)
-      end;
+		tag1 := false;
+		tag2 := false
+	      end;
+	      lprintf " %s" (string_of_server s)
+	    end;
             masters := s :: !masters
         | _ -> s.server_master <- false
   ) server_list;
@@ -780,8 +823,7 @@ let update_master_servers _ =
         masters := s :: !masters;
         masters := List.rev (List.sort compare_servers !masters);
 
-        server_send_share s.server_has_zlib sock
-          (DonkeyShare.all_shared ())
+	DonkeyGlobals.master_server := Some s;
     )
   in
 
@@ -794,7 +836,7 @@ let update_master_servers _ =
     if !nconnected_servers > max_allowed_connected_servers then
       begin
         if !verbose_location then
-            lprintf_nl "master servers: disconnect %s" (Ip.to_string s.server_ip);
+            lprintf_nl "master servers: disconnect %s" (string_of_server s);
         nconnected_servers := !nconnected_servers - 3;
         do_if_connected  s.server_sock (fun sock ->
             (* We will disconnect from this server.
@@ -813,40 +855,43 @@ let update_master_servers _ =
             - connection_last_conn s.server_connection_control
           in
           if !verbose_location then
-              lprintf_nl "master servers: Checking ip:%s, users: %Ld, ct:%d"
-          (Ip.to_string s.server_ip) s.server_nusers connection_time;
-          if not s.server_master
-            && (s.server_preferred
-                 || connection_time > !!become_master_delay
-                 || !!immediate_master
-               )
-            then
+              lprintf_nl "master servers: Checking %s, users: %Ld, ct:%d"
+		(string_of_server s)
+		(match s.server_nusers with None -> 0L | Some v -> v)
+		connection_time;
+          if not s.server_master then
               begin
-                if (!nmasters < max_allowed_connected_servers) then
-                  begin
-                    if !verbose_location then
-                        lprintf_nl "master servers: raising  %s"
-                                  (Ip.to_string s.server_ip);
+                if (!nmasters < max_allowed_connected_servers) then begin
+		  if !verbose_location then
+		    lprintf_nl "master servers: raising %s" (string_of_server s);
                     make_master s
-                  end
+		  end
                 else if s.server_sent_all_queries then
                   match !masters with
                       [] -> disconnect_old_server s
                     | ss :: tail ->
+			let ss_nusers =
+			  match ss.server_nusers with
+			    None -> 0L
+			  | Some v -> v in
+			let s_nusers =
+			  match s.server_nusers with
+			    None -> 0L
+			  | Some v -> v in
                         (* check if the non-master has more users
                            or is a preferred one *)
                         if (s.server_preferred && not ss.server_preferred)
                           || (!!keep_best_server
-                               && mini ((Int64.to_int ss.server_nusers) + 1000)
-                                       ((Int64.to_int ss.server_nusers) * 5)
-                                  < (Int64.to_int s.server_nusers)
+                               && min ((Int64.to_int ss_nusers) + 1000)
+                                       ((Int64.to_int ss_nusers) * 5)
+                                  < (Int64.to_int s_nusers)
                              )
                           then
                             begin
                               if !verbose_location then
                                 lprintf_nl
                                   "master servers: raising %s, disconnected %s"
-                                  (Ip.to_string s.server_ip) (Ip.to_string ss.server_ip);
+                                  (string_of_server s) (string_of_server ss);
                               ss.server_master <- false;
                               masters := tail;
                               make_master s
