@@ -138,7 +138,7 @@ let set_file_size file size =
       file_must_update (as_file file);
     end
 
-let new_file file_id file_name file_size =
+let new_file file_id file_name file_size user =
   let file_temp = Filename.concat !!temp_directory
       (Printf.sprintf "FileTP-%s" (Md4.to_string file_id)) in
   let t = Unix32.create_rw file_temp in
@@ -148,11 +148,12 @@ let new_file file_id file_name file_size =
 (*      file_name = file_name; *)
       file_clients = [];
       file_swarmer = None;
-      file_filenames = [Filename.basename file_name, GuiTypes.noips()];
       file_clients_queue = Queues.workflow (fun _ -> false);
       file_nconnected_clients = 0;
     } and file_impl =  {
       dummy_file_impl with
+      impl_file_owner = user;
+      impl_file_group = user.user_default_group;
       impl_file_fd = Some t;
       impl_file_size = zero;
       impl_file_downloaded = zero;
@@ -160,6 +161,7 @@ let new_file file_id file_name file_size =
       impl_file_ops = file_ops;
       impl_file_age = last_time ();
       impl_file_best_name = Filename.basename file_name;
+      impl_file_filenames = [Filename.basename file_name];
     }
   in
   set_file_size file file_size;
@@ -168,11 +170,11 @@ let new_file file_id file_name file_size =
 (*      lprintf "ADD FILE TO DOWNLOAD LIST\n"; *)
   file
 
-let new_file file_id file_name file_size =
+let new_file file_id file_name file_size users =
   try
     Hashtbl.find files_by_uid file_id
   with _ ->
-      let file = new_file file_id file_name file_size  in
+      let file = new_file file_id file_name file_size users in
       Hashtbl.add files_by_uid file_id file;
       file
 
@@ -221,7 +223,7 @@ let add_download file c url =
           download_chunks = chunks;
           download_uploader = None;
           download_ranges = [];
-          download_block = None;
+          download_blocks = [];
         }];
       file.file_clients <- c :: file.file_clients;
       file_add_source (as_file file) (as_client c);
