@@ -49,7 +49,6 @@ and file = {
 (*    mutable file_name : string;*)
     mutable file_swarmer : CommonSwarming.t option;
     mutable file_clients : client list;
-    mutable file_filenames : (string * ips_list) list;
     mutable file_clients_queue : client  Queues.Queue.t;
     mutable file_nconnected_clients : int;
   }
@@ -60,7 +59,7 @@ and download = {
     mutable download_chunks : (int64 * int64) list;
     mutable download_uploader : CommonSwarming.uploader option;
     mutable download_ranges : (int64 * int64 * CommonSwarming.range) list;
-    mutable download_block : CommonSwarming.block option;
+    mutable download_blocks : CommonSwarming.uploader_block list;
   }
 
 and tp_proto = {
