@@ -77,8 +77,8 @@ let recover_file file =
       if file_state file = FileDownloading then
         GnutellaServers.really_recover_file file        
 
-let download_file r =
-  let file = GnutellaServers.really_download_file r in
+let download_file r user =
+  let file = GnutellaServers.really_download_file r user in
   recover_file file;
   as_file file
 
@@ -211,9 +211,15 @@ that we can reuse queries *)
   network.op_network_forget_search <- (fun s ->
       Hashtbl.remove searches_by_uid (find_search s).search_uid  
   );
+  network.op_network_reset <- (fun _ -> ());
   network.op_network_connected <- (fun _ ->
       !connected_servers <> []
   );
+  network.op_network_ports <- (fun _ ->
+    [
+    !!client_port, "client_port TCP+UDP";
+    ]);
+  network.op_network_porttest_result <- (fun _ -> PorttestNotAvailable);
   network.op_network_share <- (fun fullname codedname size ->
       (*
       lprintf "*************** op_network_share %s\n
@@ -236,8 +242,8 @@ that we can reuse queries *)
         end
   );
 (* TODO RESULT *)
-  network.op_network_download <- (fun r ->
-      result_download r
+  network.op_network_download <- (fun r user ->
+      result_download r user
   )
   
 let file_num file =
@@ -283,6 +289,7 @@ let _ =
 module P = GuiTypes
   
 let _ =
+  file_ops.op_file_print <- (fun file buf -> ());
   file_ops.op_file_cancel <- (fun file ->
       CommonSwarming.remove_swarmer file.file_swarmer;
       file.file_swarmer <- None;
@@ -296,7 +303,6 @@ let _ =
   file_ops.op_file_info <- (fun file ->
       { (impl_file_info file.file_file) with
         P.file_network = network.network_num;
-        P.file_names = [file.file_name, P.noips()];
         
          P.file_chunks = (match file.file_swarmer with
          | None -> None 
@@ -365,7 +371,7 @@ let _ =
       List2.tail_map (fun s -> as_server s.server_server)
       !connected_servers
   );
-  network.op_network_parse_url <- (fun url ->
+  network.op_network_parse_url <- (fun url user ->
       match String2.split (String.escaped url) '|' with
       | "gnut://" :: "server" :: ip :: port :: _ ->  
           let ip = Ip.addr_of_string ip in
@@ -397,7 +403,7 @@ let _ =
               (* Start a download for this file *)
               let rs = new_result name size [] uids [] in
               let r = IndexedResults.get_result rs in
-              let file = download_file r in
+              let file = download_file r user in
               CommonInteractive.start_download file;
               "started Gnutella download", true
             end
