@@ -18,6 +18,7 @@
 *)
 
 open Printf2
+open Int64ops
 open CommonInteractive
 open CommonFile
 open CommonComplexOptions
@@ -68,8 +69,8 @@ let file_complete file =
   (try file_completed (as_file file.file_file)
     with e ->
         lprintf "Exception %s in file completed"
-        (Printexc2.to_string e)
-        ; lprint_newline ());
+        (Printexc2.to_string e);
+        lprint_newline ());
   current_files := List2.removeq file !current_files;
   old_files =:= (file.file_name, file_size file) :: !!old_files;
   List.iter (fun c ->
@@ -145,7 +146,7 @@ let read_stream c file sock b =
       Unix2.really_write fd b.buf b.pos b.len;
     end; *)
 (*      lprintf "DIFF %d/%d" nread b.len; lprint_newline ();*)
-    c.client_pos <- Int64.add c.client_pos (Int64.of_int b.len);
+    c.client_pos <- c.client_pos ++ (Int64.of_int b.len);
 (*
       lprintf "NEW SOURCE POS %s" (Int64.to_string c.source_pos);
 lprint_newline ();
@@ -153,7 +154,7 @@ lprint_newline ();
     TcpBufferedSocket.buf_used b b.len;
 (*    if c.client_pos > file_downloaded file then begin
         add_file_downloaded file.file_file
-        (Int64.sub c.client_pos (file_downloaded file))
+        (c.client_pos -- (file_downloaded file))
       end; *)
     if file_downloaded file = file_size file then
       
@@ -356,4 +357,4 @@ let listen () =
       lprintf "Exception %s while init limewire server" 
         (Printexc2.to_string e);
       lprint_newline ()
-      
\ No newline at end of file
+      
