@@ -95,9 +95,6 @@ val starts_with : (* string *) string -> (* start *) string -> bool
 (* [replace_char s c1 c2] replaces char [c1] by char [c2] in [s] *)
 val replace_char : string -> char -> char -> unit
 
-(* [shorten max s] cuts [s] to [max] chars if [s] is longer than [max] *)
-val shorten : int -> string -> string
-  
 (* [stem s] cuts the string [s] in small words, for indexation eg *)
 val stem : string -> string list
   
